@@ -26,9 +26,7 @@
  *
  */
 
-#ifndef ANDROID
-#define _GNU_SOURCE
-#else
+#ifdef HAVE_LIBGEN_H
 #include <libgen.h>
 #endif
 #include <stdio.h>
@@ -57,6 +55,9 @@
 #include <limits.h>
 #include <locale.h>
 #include <uwildmat/uwildmat.h>
+#ifdef HAVE_GLIB
+#include <glib.h>
+#endif
 
 #include "drmtest.h"
 #include "intel_chipset.h"
@@ -68,6 +69,7 @@
 #include "igt_core.h"
 #include "igt_aux.h"
 #include "igt_sysfs.h"
+#include "igt_rc.h"
 
 #ifdef HAVE_LIBGEN_H
 #include <libgen.h>   /* for basename() on Solaris */
@@ -225,6 +227,27 @@
  * - 'basic*,advanced*' match any subtest starting basic or advanced
  * - '*,!basic*' match any subtest not starting basic
  * - 'basic*,!basic-render*' match any subtest starting basic but not starting basic-render
+ *
+ * # Configuration
+ *
+ * Some of IGT's behavior can be configured through a configuration file.
+ * By default, this file is expected to exist in ~/.igtrc . The directory for
+ * this can be overriden by setting the environment variable %IGT_CONFIG_PATH.
+ * An example configuration follows:
+ *
+ * |[<!-- language="plain" -->
+ *	&num; The common configuration section follows.
+ *	[Common]
+ *	FrameDumpPath=/tmp # The path to dump frames that fail comparison checks
+ *
+ *	&num; The following section is used for configuring the Device Under Test.
+ *	&num; It is not mandatory and allows overriding default values.
+ *	[DUT]
+ *	SuspendResumeDelay=10
+ * ]|
+ *
+ * Some specific configuration options may be used by specific parts of IGT,
+ * such as those related to Chamelium support.
  */
 
 static unsigned int exit_handler_count;
@@ -271,6 +294,12 @@ static struct {
 } log_buffer;
 static pthread_mutex_t log_buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+#ifdef HAVE_GLIB
+GKeyFile *igt_key_file;
+#endif
+
+char *igt_frame_dump_path;
+
 const char *igt_test_name(void)
 {
 	return command_str;
@@ -329,6 +358,30 @@ static void _igt_log_buffer_dump(void)
 	pthread_mutex_unlock(&log_buffer_mutex);
 }
 
+/**
+ * igt_log_buffer_inspect:
+ *
+ * Provides a way to replay the internal igt log buffer for inspection.
+ * @check: A user-specified handler that gets invoked for each line of
+ *         the log buffer. The handler should return true to stop
+ *         inspecting the rest of the buffer.
+ * @data: passed as a user argument to the inspection function.
+ */
+void igt_log_buffer_inspect(igt_buffer_log_handler_t check, void *data)
+{
+	uint8_t i;
+	pthread_mutex_lock(&log_buffer_mutex);
+
+	i = log_buffer.start;
+	do {
+		if (check(log_buffer.entries[i], data))
+			break;
+		i++;
+	} while (i != log_buffer.start && i != log_buffer.end);
+
+	pthread_mutex_unlock(&log_buffer_mutex);
+}
+
 __attribute__((format(printf, 1, 2)))
 static void kmsg(const char *format, ...)
 #define KERN_EMER	"<0>"
@@ -536,9 +589,9 @@ static void print_version(void)
 
 	uname(&uts);
 
-	fprintf(stdout, "IGT-Version: %s-%s (%s) (%s: %s %s)\n", PACKAGE_VERSION,
-		IGT_GIT_SHA1, TARGET_CPU_PLATFORM,
-		uts.sysname, uts.release, uts.machine);
+	igt_info("IGT-Version: %s-%s (%s) (%s: %s %s)\n", PACKAGE_VERSION,
+		 IGT_GIT_SHA1, TARGET_CPU_PLATFORM,
+		 uts.sysname, uts.release, uts.machine);
 }
 
 static void print_usage(const char *help_str, bool output_on_stderr)
@@ -569,6 +622,84 @@ static void oom_adjust_for_doom(void)
 
 }
 
+#ifdef HAVE_GLIB
+static void common_init_config(void)
+{
+	char *key_file_env = NULL;
+	char *key_file_loc = NULL;
+	GError *error = NULL;
+	int ret;
+
+	/* Determine igt config path */
+	key_file_env = getenv("IGT_CONFIG_PATH");
+	if (key_file_env) {
+		key_file_loc = key_file_env;
+	} else {
+		key_file_loc = malloc(100);
+		snprintf(key_file_loc, 100, "%s/.igtrc", g_get_home_dir());
+	}
+
+	/* Load igt config file */
+	igt_key_file = g_key_file_new();
+	ret = g_key_file_load_from_file(igt_key_file, key_file_loc,
+					G_KEY_FILE_NONE, &error);
+	if (error && error->code == G_KEY_FILE_ERROR) {
+		g_error_free(error);
+		g_key_file_free(igt_key_file);
+		igt_key_file = NULL;
+
+		goto out;
+	}
+
+	g_clear_error(&error);
+
+	if (!igt_frame_dump_path)
+		igt_frame_dump_path =
+			g_key_file_get_string(igt_key_file, "Common",
+					      "FrameDumpPath", &error);
+
+	g_clear_error(&error);
+
+	ret = g_key_file_get_integer(igt_key_file, "DUT", "SuspendResumeDelay",
+				     &error);
+	assert(!error || error->code != G_KEY_FILE_ERROR_INVALID_VALUE);
+
+	g_clear_error(&error);
+
+	if (ret != 0)
+		igt_set_autoresume_delay(ret);
+
+out:
+	if (!key_file_env && key_file_loc)
+		free(key_file_loc);
+}
+#endif
+
+static void common_init_env(void)
+{
+	const char *env;
+
+	if (!isatty(STDOUT_FILENO) || getenv("IGT_PLAIN_OUTPUT"))
+		__igt_plain_output = true;
+
+	if (!__igt_plain_output)
+		setlocale(LC_ALL, "");
+
+	env = getenv("IGT_LOG_LEVEL");
+	if (env) {
+		if (strcmp(env, "debug") == 0)
+			igt_log_level = IGT_LOG_DEBUG;
+		else if (strcmp(env, "info") == 0)
+			igt_log_level = IGT_LOG_INFO;
+		else if (strcmp(env, "warn") == 0)
+			igt_log_level = IGT_LOG_WARN;
+		else if (strcmp(env, "none") == 0)
+			igt_log_level = IGT_LOG_NONE;
+	}
+
+	igt_frame_dump_path = getenv("IGT_FRAME_DUMP_PATH");
+}
+
 static int common_init(int *argc, char **argv,
 		       const char *extra_short_opts,
 		       const struct option *extra_long_opts,
@@ -592,25 +723,8 @@ static int common_init(int *argc, char **argv,
 	int extra_opt_count;
 	int all_opt_count;
 	int ret = 0;
-	const char *env;
 
-	if (!isatty(STDOUT_FILENO) || getenv("IGT_PLAIN_OUTPUT"))
-		__igt_plain_output = true;
-
-	if (!__igt_plain_output)
-		setlocale(LC_ALL, "");
-
-	env = getenv("IGT_LOG_LEVEL");
-	if (env) {
-		if (strcmp(env, "debug") == 0)
-			igt_log_level = IGT_LOG_DEBUG;
-		else if (strcmp(env, "info") == 0)
-			igt_log_level = IGT_LOG_INFO;
-		else if (strcmp(env, "warn") == 0)
-			igt_log_level = IGT_LOG_WARN;
-		else if (strcmp(env, "none") == 0)
-			igt_log_level = IGT_LOG_NONE;
-	}
+	common_init_env();
 
 	command_str = argv[0];
 	if (strrchr(command_str, '/'))
@@ -713,6 +827,10 @@ static int common_init(int *argc, char **argv,
 		}
 	}
 
+#ifdef HAVE_GLIB
+	common_init_config();
+#endif
+
 out:
 	free(short_opts);
 	free(combined_opts);
@@ -737,6 +855,7 @@ out:
 		kmsg(KERN_INFO "[IGT] %s: executing\n", command_str);
 		print_version();
 
+		sync();
 		oom_adjust_for_doom();
 		low_mem_killer_disable(true);
 	}
@@ -911,10 +1030,10 @@ static void exit_subtest(const char *result)
 	struct timespec now;
 
 	gettime(&now);
-	printf("%sSubtest %s: %s (%.3fs)%s\n",
-	       (!__igt_plain_output) ? "\x1b[1m" : "",
-	       in_subtest, result, time_elapsed(&subtest_time, &now),
-	       (!__igt_plain_output) ? "\x1b[0m" : "");
+	igt_info("%sSubtest %s: %s (%.3fs)%s\n",
+		 (!__igt_plain_output) ? "\x1b[1m" : "",
+		 in_subtest, result, time_elapsed(&subtest_time, &now),
+		 (!__igt_plain_output) ? "\x1b[0m" : "");
 	fflush(stdout);
 
 	in_subtest = NULL;
@@ -1053,7 +1172,7 @@ void igt_fail(int exitcode)
 		else
 			exit_subtest("FAIL");
 	} else {
-		assert(!test_with_subtests || in_fixture);
+		assert(igt_can_fail());
 
 		if (in_fixture) {
 			skip_subtests_henceforth = FAIL;
@@ -1064,6 +1183,22 @@ void igt_fail(int exitcode)
 	}
 }
 
+/**
+ * igt_can_fail:
+ *
+ * Returns true if called from either an #igt_fixture, #igt_subtest or a
+ * testcase without subtests, i.e. #igt_simple_main. Returns false otherwise. In
+ * other words, it checks whether it's legal to call igt_fail(), igt_skip_on()
+ * and all the convenience macros build around those.
+ *
+ * This is useful to make sure that library code is called from the right
+ * places.
+ */
+bool igt_can_fail(void)
+{
+	return !test_with_subtests || in_fixture || in_subtest;
+}
+
 static bool run_under_gdb(void)
 {
 	char buf[1024];
@@ -1083,7 +1218,7 @@ static void print_backtrace(void)
 	unw_context_t uc;
 	int stack_num = 0;
 
-	printf("Stack trace:\n");
+	igt_info("Stack trace:\n");
 
 	unw_getcontext(&uc);
 	unw_init_local(&cursor, &uc);
@@ -1094,8 +1229,8 @@ static void print_backtrace(void)
 		if (unw_get_proc_name(&cursor, name, 255, &off) < 0)
 			strcpy(name, "<unknown>");
 
-		printf("  #%d [%s+0x%x]\n", stack_num++, name,
-		       (unsigned int) off);
+		igt_info("  #%d [%s+0x%x]\n", stack_num++, name,
+			 (unsigned int) off);
 	}
 }
 
@@ -1321,6 +1456,11 @@ void igt_exit(void)
 {
 	igt_exit_called = true;
 
+#ifdef HAVE_GLIB
+	if (igt_key_file)
+		g_key_file_free(igt_key_file);
+#endif
+
 	if (run_single_subtest && !run_single_subtest_found) {
 		igt_warn("Unknown subtest: %s\n", run_single_subtest);
 		exit(IGT_EXIT_INVALID);
@@ -1557,6 +1697,32 @@ bool __igt_fork(void)
 
 }
 
+/**
+ * igt_child_done:
+ *
+ * Lets the IGT core know that one of the children has exited.
+ */
+void igt_child_done(pid_t pid)
+{
+	int i = 0;
+	int found = -1;
+
+	igt_assert(num_test_children > 1);
+
+	for (i = 0; i < num_test_children; i++) {
+		if (pid == test_children[i]) {
+			found = i;
+			break;
+		}
+	}
+
+	igt_assert(found >= 0);
+
+	num_test_children--;
+	for (i = found; i < num_test_children; i++)
+		test_children[i] = test_children[i + 1];
+}
+
 /**
  * igt_waitchildren:
  *
@@ -1867,7 +2033,7 @@ void igt_skip_on_simulation(void)
 	if (igt_only_list_subtests())
 		return;
 
-	if (!in_fixture && !in_subtest) {
+	if (!igt_can_fail()) {
 		igt_fixture
 			igt_require(!igt_run_in_simulation());
 	} else
@@ -2073,3 +2239,125 @@ FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
 
 	return fp;
 }
+
+static void log_output(int *fd, enum igt_log_level level)
+{
+	ssize_t len;
+	char buf[PIPE_BUF];
+
+	if (*fd < 0)
+		return;
+
+	memset(buf, 0, sizeof(buf));
+	len = read(*fd, buf, sizeof(buf));
+	if (len <= 0) {
+		close(*fd);
+		*fd = -1;
+		return;
+	}
+
+	igt_log(IGT_LOG_DOMAIN, level, "[cmd] %s", buf);
+}
+
+/**
+ * igt_system:
+ *
+ * An improved replacement of the system() call.
+ *
+ * Executes the shell command specified in @command with the added feature of
+ * concurrently capturing its stdout and stderr to igt_log and igt_warn
+ * respectively.
+ *
+ * Returns: The exit status of the executed process. -1 for failure.
+ */
+int igt_system(const char *command)
+{
+	int outpipe[2] = { -1, -1 };
+	int errpipe[2] = { -1, -1 };
+	int status;
+	struct igt_helper_process process = {};
+
+	if (pipe(outpipe) < 0)
+		goto err;
+	if (pipe(errpipe) < 0)
+		goto err;
+
+	igt_fork_helper(&process) {
+		close(outpipe[0]);
+		close(errpipe[0]);
+
+		if (dup2(outpipe[1], STDOUT_FILENO) < 0)
+			goto child_err;
+		if (dup2(errpipe[1], STDERR_FILENO) < 0)
+			goto child_err;
+
+		execl("/bin/sh", "sh", "-c", command,
+		      (char *) NULL);
+
+	child_err:
+		exit(EXIT_FAILURE);
+	}
+
+	close(outpipe[1]);
+	close(errpipe[1]);
+
+	while (outpipe[0] >= 0 || errpipe[0] >= 0) {
+		log_output(&outpipe[0], IGT_LOG_INFO);
+		log_output(&errpipe[0], IGT_LOG_WARN);
+	}
+
+	status = igt_wait_helper(&process);
+
+	return WEXITSTATUS(status);
+err:
+	close(outpipe[0]);
+	close(outpipe[1]);
+	close(errpipe[0]);
+	close(errpipe[1]);
+	return -1;
+}
+
+/**
+ * igt_system_quiet:
+ * Similar to igt_system(), except redirect output to /dev/null
+ *
+ * Returns: The exit status of the executed process. -1 for failure.
+ */
+int igt_system_quiet(const char *command)
+{
+	int stderr_fd_copy, stdout_fd_copy, status, nullfd;
+
+	/* redirect */
+	if ((nullfd = open("/dev/null", O_WRONLY)) == -1)
+		goto err;
+	if ((stdout_fd_copy = dup(STDOUT_FILENO)) == -1)
+		goto err;
+	if ((stderr_fd_copy = dup(STDERR_FILENO)) == -1)
+		goto err;
+
+	if (dup2(nullfd, STDOUT_FILENO) == -1)
+		goto err;
+	if (dup2(nullfd, STDERR_FILENO) == -1)
+		goto err;
+
+	if ((status = system(command)) == -1)
+		goto err;
+
+	/* restore */
+	if (dup2(stdout_fd_copy, STDOUT_FILENO) == -1)
+		goto err;
+	if (dup2(stderr_fd_copy, STDERR_FILENO) == -1)
+		goto err;
+
+	close(stdout_fd_copy);
+	close(stderr_fd_copy);
+	close(nullfd);
+
+	return WEXITSTATUS(status);
+err:
+	close(stderr_fd_copy);
+	close(stdout_fd_copy);
+	close(nullfd);
+
+	return -1;
+}
