@@ -25,6 +25,7 @@
 #include <inttypes.h>
 #include <sys/stat.h>
 #include <sys/mount.h>
+#include <sys/sysmacros.h>
 #include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
@@ -126,6 +127,73 @@ const char *igt_debugfs_mount(void)
 	return "/sys/kernel/debug";
 }
 
+/**
+ * igt_debugfs_path:
+ * @device: fd of the device
+ * @path: buffer to store path
+ * @pathlen: len of @path buffer.
+ *
+ * This finds the debugfs directory corresponding to @device.
+ *
+ * Returns:
+ * The directory path, or NULL on failure.
+ */
+char *igt_debugfs_path(int device, char *path, int pathlen)
+{
+	struct stat st;
+	const char *debugfs_root;
+	int idx;
+
+	if (fstat(device, &st)) {
+		igt_debug("Couldn't stat FD for DRM device: %s\n", strerror(errno));
+		return NULL;
+	}
+
+	if (!S_ISCHR(st.st_mode)) {
+		igt_debug("FD for DRM device not a char device!\n");
+		return NULL;
+	}
+
+	debugfs_root = igt_debugfs_mount();
+
+	idx = minor(st.st_rdev);
+	snprintf(path, pathlen, "%s/dri/%d/name", debugfs_root, idx);
+	if (stat(path, &st))
+		return NULL;
+
+	if (idx >= 64) {
+		int file, name_len, cmp_len;
+		char name[100], cmp[100];
+
+		file = open(path, O_RDONLY);
+		if (file < 0)
+			return NULL;
+
+		name_len = read(file, name, sizeof(name));
+		close(file);
+
+		for (idx = 0; idx < 16; idx++) {
+			snprintf(path, pathlen, "%s/dri/%d/name",
+				 debugfs_root, idx);
+			file = open(path, O_RDONLY);
+			if (file < 0)
+				return NULL;
+
+			cmp_len = read(file, cmp, sizeof(cmp));
+			close(file);
+
+			if (cmp_len == name_len && !memcmp(cmp, name, name_len))
+				break;
+		}
+
+		if (idx == 16)
+			return NULL;
+	}
+
+	snprintf(path, pathlen, "%s/dri/%d", debugfs_root, idx);
+	return path;
+}
+
 /**
  * igt_debugfs_dir:
  * @device: fd of the device
@@ -138,58 +206,11 @@ const char *igt_debugfs_mount(void)
  */
 int igt_debugfs_dir(int device)
 {
-	struct stat st;
-	const char *debugfs_root;
 	char path[200];
-	int idx;
 
-	if (fstat(device, &st)) {
-		igt_debug("Couldn't stat FD for DRM device: %s\n", strerror(errno));
-		return -1;
-	}
-
-	if (!S_ISCHR(st.st_mode)) {
-		igt_debug("FD for DRM device not a char device!\n");
-		return -1;
-	}
-
-	debugfs_root = igt_debugfs_mount();
-
-	idx = minor(st.st_rdev);
-	snprintf(path, sizeof(path), "%s/dri/%d/name", debugfs_root, idx);
-	if (stat(path, &st))
+	if (!igt_debugfs_path(device, path, sizeof(path)))
 		return -1;
 
-	if (idx >= 64) {
-		int file, name_len, cmp_len;
-		char name[100], cmp[100];
-
-		file = open(path, O_RDONLY);
-		if (file < 0)
-			return -1;
-
-		name_len = read(file, name, sizeof(name));
-		close(file);
-
-		for (idx = 0; idx < 16; idx++) {
-			snprintf(path, sizeof(path), "%s/dri/%d/name",
-				 debugfs_root, idx);
-			file = open(path, O_RDONLY);
-			if (file < 0)
-				return -1;
-
-			cmp_len = read(file, cmp, sizeof(cmp));
-			close(file);
-
-			if (cmp_len == name_len && !memcmp(cmp, name, name_len))
-				break;
-		}
-
-		if (idx == 16)
-			return -1;
-	}
-
-	snprintf(path, sizeof(path), "%s/dri/%d", debugfs_root, idx);
 	igt_debug("Opening debugfs directory '%s'\n", path);
 	return open(path, O_RDONLY);
 }
@@ -281,6 +302,26 @@ bool igt_debugfs_search(int device, const char *filename, const char *substring)
  * Pipe CRC
  */
 
+static bool igt_find_crc_mismatch(const igt_crc_t *a, const igt_crc_t *b,
+				  int *index)
+{
+	int i;
+
+	if (a->n_words != b->n_words)
+		return true;
+
+	for (i = 0; i < a->n_words; i++) {
+		if (a->crc[i] != b->crc[i]) {
+			if (index)
+				*index = i;
+
+			return true;
+		}
+	}
+
+	return false;
+}
+
 /**
  * igt_assert_crc_equal:
  * @a: first pipe CRC value
@@ -294,31 +335,82 @@ bool igt_debugfs_search(int device, const char *filename, const char *substring)
  */
 void igt_assert_crc_equal(const igt_crc_t *a, const igt_crc_t *b)
 {
-	int i;
+	int index;
+	bool mismatch;
 
-	for (i = 0; i < a->n_words; i++)
-		igt_assert_eq_u32(a->crc[i], b->crc[i]);
+	mismatch = igt_find_crc_mismatch(a, b, &index);
+	if (mismatch)
+		igt_debug("CRC mismatch at index %d: 0x%x != 0x%x\n", index,
+			  a->crc[index], b->crc[index]);
+
+	igt_assert(!mismatch);
+}
+
+/**
+ * igt_check_crc_equal:
+ * @a: first pipe CRC value
+ * @b: second pipe CRC value
+ *
+ * Compares two CRC values and return whether they match.
+ *
+ * Returns: A boolean indicating whether the CRC values match
+ */
+bool igt_check_crc_equal(const igt_crc_t *a, const igt_crc_t *b)
+{
+	int index;
+	bool mismatch;
+
+	mismatch = igt_find_crc_mismatch(a, b, &index);
+	if (mismatch)
+		igt_debug("CRC mismatch at index %d: 0x%x != 0x%x\n", index,
+			  a->crc[index], b->crc[index]);
+
+	return !mismatch;
+}
+
+/**
+ * igt_crc_to_string_extended:
+ * @crc: pipe CRC value to print
+ * @delimiter: The delimiter to use between crc words
+ * @crc_size: the number of bytes to print per crc word (between 1 and 4)
+ *
+ * This function allocates a string and formats @crc into it, depending on
+ * @delimiter and @crc_size.
+ * The caller is responsible for freeing the string.
+ *
+ * This should only ever be used for diagnostic debug output.
+ */
+char *igt_crc_to_string_extended(igt_crc_t *crc, char delimiter, int crc_size)
+{
+	int i;
+	int len = 0;
+	int field_width = 2 * crc_size; /* Two chars per byte. */
+	char *buf = malloc((field_width+1) * crc->n_words * sizeof(char));
+
+	if (!buf)
+		return NULL;
+
+	for (i = 0; i < crc->n_words; i++)
+		len += sprintf(buf + len, "%0*x%c", field_width,
+			       crc->crc[i], delimiter);
+
+	/* Eat the last delimiter */
+	buf[len - 1] = '\0';
+	return buf;
 }
 
 /**
  * igt_crc_to_string:
  * @crc: pipe CRC value to print
  *
- * This formats @crc into a string buffer which is owned by igt_crc_to_string().
- * The next call will override the buffer again, which makes this multithreading
- * unsafe.
+ * This function allocates a string and formats @crc into it.
+ * The caller is responsible for freeing the string.
  *
  * This should only ever be used for diagnostic debug output.
  */
 char *igt_crc_to_string(igt_crc_t *crc)
 {
-	int i;
-	char buf[128] = { 0 };
-
-	for (i = 0; i < crc->n_words; i++)
-		sprintf(buf + strlen(buf), "%08x ", crc->crc[i]);
-
-	return strdup(buf);
+	return igt_crc_to_string_extended(crc, ' ', 4);
 }
 
 #define MAX_CRC_ENTRIES 10
@@ -420,7 +512,7 @@ static void igt_pipe_crc_reset(int drm_fd)
 	}
 
 	while ((dirent = readdir(dir))) {
-		char buf[128];
+		char buf[PATH_MAX];
 
 		if (strcmp(dirent->d_name, "crtc-") != 0)
 			continue;
@@ -454,7 +546,7 @@ static void igt_pipe_crc_reset(int drm_fd)
 static void pipe_crc_exit_handler(int sig)
 {
 	struct dirent *dirent;
-	char buf[128];
+	char buf[PATH_MAX];
 	DIR *dir;
 	int fd;
 
@@ -508,7 +600,7 @@ void igt_require_pipe_crc(int fd)
 
 static void igt_hpd_storm_exit_handler(int sig)
 {
-	int fd = drm_open_driver_master(DRIVER_INTEL);
+	int fd = drm_open_driver(DRIVER_INTEL);
 
 	/* Here we assume that only one i915 device will be ever present */
 	igt_hpd_storm_reset(fd);
