@@ -21,13 +21,13 @@
  * IN THE SOFTWARE.
  */
 
-#include "igt.h"
+#include <signal.h>
+#include <errno.h>
+
 #include "igt_core.h"
 #include "igt_sysfs.h"
 #include "igt_kmod.h"
 
-#include <signal.h>
-
 /**
  * SECTION:igt_kmod
  * @short_description: Wrappers around libkmod for module loading/unloading
@@ -56,14 +56,50 @@ static void squelch(void *data, int priority,
 static struct kmod_ctx *kmod_ctx(void)
 {
 	static struct kmod_ctx *ctx;
+	const char **config_paths = NULL;
+	char *config_paths_str;
+	char *dirname;
 
-	if (!ctx) {
-		ctx = kmod_new(NULL, NULL);
-		igt_assert(ctx != NULL);
+	if (ctx)
+		goto out;
 
-		kmod_set_log_fn(ctx, squelch, NULL);
+	dirname = getenv("IGT_KMOD_DIRNAME");
+	if (dirname)
+		igt_debug("kmod dirname = %s\n", dirname);
+
+	config_paths_str = getenv("IGT_KMOD_CONFIG_PATHS");
+	if (config_paths_str)
+		igt_debug("kmod config paths = %s\n", config_paths_str);
+
+	if (config_paths_str) {
+		unsigned count = !!strlen(config_paths_str);
+		unsigned i;
+		char* p;
+
+		p = config_paths_str;
+		while ((p = strchr(p, ':'))) p++, count++;
+
+
+		config_paths = malloc(sizeof(*config_paths) * (count + 1));
+		igt_assert(config_paths != NULL);
+
+		p = config_paths_str;
+		for (i = 0; i < count; ++i) {
+			config_paths[i] = p;
+
+			if ((p = strchr(p, ':')))
+				*p++ = '\0';
+		}
+		config_paths[i] = NULL;
 	}
 
+	ctx = kmod_new(dirname, config_paths);
+	igt_assert(ctx != NULL);
+
+	free(config_paths);
+
+	kmod_set_log_fn(ctx, squelch, NULL);
+out:
 	return ctx;
 }
 
@@ -155,7 +191,7 @@ igt_kmod_load(const char *mod_name, const char *opts)
 	}
 out:
 	kmod_module_unref(kmod);
-	return -err ? err < 0 : err;
+	return err < 0 ? err : 0;
 }
 
 
@@ -192,7 +228,7 @@ igt_kmod_unload(const char *mod_name, unsigned int flags)
 
 out:
 	kmod_module_unref(kmod);
-	return -err ? err < 0 : err;
+	return err < 0 ? err : 0;
 }
 
 /**
