@@ -186,7 +186,8 @@ const char *igt_crtc_prop_names[IGT_NUM_CRTC_PROPS] = {
 
 const char *igt_connector_prop_names[IGT_NUM_CONNECTOR_PROPS] = {
 	"scaling mode",
-	"CRTC_ID"
+	"CRTC_ID",
+	"DPMS"
 };
 
 /*
@@ -319,12 +320,12 @@ const unsigned char* igt_kms_get_alt_edid(void)
  */
 const char *kmstest_pipe_name(enum pipe pipe)
 {
-	const char *str[] = { "A", "B", "C" };
+	const char *str[] = { "A", "B", "C", "D", "E", "F"};
 
 	if (pipe == PIPE_NONE)
 		return "None";
 
-	if (pipe > 2)
+	if (pipe >= IGT_MAX_PIPES)
 		return "invalid";
 
 	return str[pipe];
@@ -344,6 +345,12 @@ int kmstest_pipe_to_index(char pipe)
 		return 1;
 	else if (pipe == 'C')
 		return 2;
+	else if (pipe == 'D')
+		return 3;
+	else if (pipe == 'E')
+		return 4;
+	else if (pipe == 'F')
+		return 5;
 	else
 		return -EINVAL;
 }
@@ -1554,9 +1561,10 @@ static void igt_display_log_shift(igt_display_t *display, int shift)
 static void igt_output_refresh(igt_output_t *output)
 {
 	igt_display_t *display = output->display;
-	unsigned long crtc_idx_mask;
+	unsigned long crtc_idx_mask = 0;
 
-	crtc_idx_mask = output->pending_crtc_idx_mask;
+	if (output->pending_pipe != PIPE_NONE)
+		crtc_idx_mask = 1 << output->pending_pipe;
 
 	kmstest_free_connector_config(&output->config);
 
@@ -1580,14 +1588,8 @@ static void igt_output_refresh(igt_output_t *output)
 						    BROADCAST_RGB_FULL);
 	}
 
-	if (output->use_override_mode)
-		output->config.default_mode = output->override_mode;
-
-	if (output->config.pipe == PIPE_NONE)
-		return;
-
 	LOG(display, "%s: Selecting pipe %s\n", output->name,
-	    kmstest_pipe_name(output->config.pipe));
+	    kmstest_pipe_name(output->pending_pipe));
 }
 
 static bool
@@ -1695,6 +1697,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		pipe->plane_cursor = -1;
 		pipe->plane_primary = -1;
 		pipe->planes = NULL;
+		pipe->out_fence_fd = -1;
 
 		get_crtc_property(display->drm_fd, pipe->crtc_id,
 				    "background_color",
@@ -1825,7 +1828,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		 * a pipe is set with igt_output_set_pipe().
 		 */
 		output->force_reprobe = true;
-		output->pending_crtc_idx_mask = 0;
+		output->pending_pipe = PIPE_NONE;
 		output->id = resources->connectors[i];
 		output->display = display;
 
@@ -1845,6 +1848,30 @@ int igt_display_get_n_pipes(igt_display_t *display)
 	return display->n_pipes;
 }
 
+void igt_display_require_output(igt_display_t *display)
+{
+	enum pipe pipe;
+	igt_output_t *output;
+
+	for_each_pipe_with_valid_output(display, pipe, output)
+		return;
+
+	igt_skip("No valid crtc/connector combinations found.\n");
+}
+
+void igt_display_require_output_on_pipe(igt_display_t *display, enum pipe pipe)
+{
+	igt_output_t *output;
+
+	igt_skip_on_f(igt_display_get_n_pipes(display) < pipe,
+		      "Pipe %s does not exist.\n", kmstest_pipe_name(pipe));
+
+	for_each_valid_output_on_pipe(display, pipe, output)
+		return;
+
+	igt_skip("No valid connector found on pipe %s\n", kmstest_pipe_name(pipe));
+}
+
 /**
  * igt_output_from_connector:
  * @display: a pointer to an #igt_display_t structure
@@ -1886,6 +1913,9 @@ static void igt_pipe_fini(igt_pipe_t *pipe)
 
 	free(pipe->planes);
 	pipe->planes = NULL;
+
+	if (pipe->out_fence_fd != -1)
+		close(pipe->out_fence_fd);
 }
 
 static void igt_output_fini(igt_output_t *output)
@@ -1928,10 +1958,12 @@ static void igt_display_refresh(igt_display_t *display)
 	for (i = 0; i < display->n_outputs; i++) {
 		output = &display->outputs[i];
 
-		if (pipes_in_use & output->pending_crtc_idx_mask)
-			goto report_dup;
+		if (output->pending_pipe != PIPE_NONE) {
+			if (pipes_in_use & (1 << output->pending_pipe))
+				goto report_dup;
 
-		pipes_in_use |= output->pending_crtc_idx_mask;
+			pipes_in_use |= 1 << output->pending_pipe;
+		}
 
 		if (output->force_reprobe)
 			igt_output_refresh(output);
@@ -1943,11 +1975,11 @@ report_dup:
 	for (; i > 0; i--) {
 		igt_output_t *b = &display->outputs[i - 1];
 
-		igt_assert_f(output->pending_crtc_idx_mask !=
-			     b->pending_crtc_idx_mask,
+		igt_assert_f(output->pending_pipe !=
+			     b->pending_pipe,
 			     "%s and %s are both trying to use pipe %s\n",
 			     igt_output_name(output), igt_output_name(b),
-			     kmstest_pipe_name(ffs(b->pending_crtc_idx_mask) - 1));
+			     kmstest_pipe_name(output->pending_pipe));
 	}
 }
 
@@ -1956,7 +1988,7 @@ static igt_pipe_t *igt_output_get_driving_pipe(igt_output_t *output)
 	igt_display_t *display = output->display;
 	enum pipe pipe;
 
-	if (!output->pending_crtc_idx_mask) {
+	if (output->pending_pipe == PIPE_NONE) {
 		/*
 		 * The user hasn't specified a pipe to use, return none.
 		 */
@@ -1966,7 +1998,7 @@ static igt_pipe_t *igt_output_get_driving_pipe(igt_output_t *output)
 		 * Otherwise, return the pending pipe (ie the pipe that should
 		 * drive this output after the commit()
 		 */
-		pipe = ffs(output->pending_crtc_idx_mask) - 1;
+		pipe = output->pending_pipe;
 	}
 
 	igt_assert(pipe >= 0 && pipe < display->n_pipes);
@@ -2004,9 +2036,9 @@ igt_plane_t *igt_pipe_get_plane_type(igt_pipe_t *pipe, int plane_type)
 		break;
 	}
 
-	igt_assert_f(plane_idx >= 0 && plane_idx < pipe->n_planes,
-		"Valid pipe->planes idx not found. plane_idx=%d plane_type=%d n_planes=%d\n",
-		plane_idx, plane_type, pipe->n_planes);
+	igt_require_f(plane_idx >= 0 && plane_idx < pipe->n_planes,
+		      "Valid pipe->planes idx not found. plane_idx=%d plane_type=%d n_planes=%d\n",
+		      plane_idx, plane_type, pipe->n_planes);
 
 	return &pipe->planes[plane_idx];
 }
@@ -2019,7 +2051,7 @@ static igt_output_t *igt_pipe_get_output(igt_pipe_t *pipe)
 	for (i = 0; i < display->n_outputs; i++) {
 		igt_output_t *output = &display->outputs[i];
 
-		if (output->pending_crtc_idx_mask == (1 << pipe->pipe))
+		if (output->pending_pipe == pipe->pipe)
 			return output;
 	}
 
@@ -2300,11 +2332,11 @@ static int igt_primary_plane_commit_legacy(igt_plane_t *primary,
 	igt_assert(!primary->rotation_changed);
 
 	if (!primary->fb_changed && !primary->position_changed &&
-	    !primary->size_changed)
+	    !primary->size_changed && !primary->pipe->mode_changed)
 		return 0;
 
 	crtc_id = pipe->crtc_id;
-	fb_id = igt_plane_get_fb_id(primary);
+	fb_id = output ? igt_plane_get_fb_id(primary) : 0;
 	if (fb_id)
 		mode = igt_output_get_mode(output);
 	else
@@ -2343,8 +2375,6 @@ static int igt_primary_plane_commit_legacy(igt_plane_t *primary,
 
 	CHECK_RETURN(ret, fail_on_error);
 
-	primary->pipe->enabled = (fb_id != 0);
-
 	return 0;
 }
 
@@ -2382,10 +2412,8 @@ static int igt_pipe_commit(igt_pipe_t *pipe,
 			   enum igt_commit_style s,
 			   bool fail_on_error)
 {
-	igt_display_t *display = pipe->display;
 	int i;
 	int ret;
-	bool need_wait_for_vblank = false;
 
 	if (pipe->background_changed) {
 		igt_crtc_set_property(pipe, pipe->background_property,
@@ -2404,21 +2432,10 @@ static int igt_pipe_commit(igt_pipe_t *pipe,
 	for (i = 0; i < pipe->n_planes; i++) {
 		igt_plane_t *plane = &pipe->planes[i];
 
-		if (plane->fb_changed || plane->position_changed || plane->size_changed)
-			need_wait_for_vblank = true;
-
 		ret = igt_plane_commit(plane, pipe, s, fail_on_error);
 		CHECK_RETURN(ret, fail_on_error);
 	}
 
-	/*
-	 * If the crtc is enabled, wait until the next vblank before returning
-	 * if we made changes to any of the planes.
-	 */
-	if (need_wait_for_vblank && pipe->enabled) {
-		igt_wait_for_vblank(display->drm_fd, pipe->pipe);
-	}
-
 	return 0;
 }
 
@@ -2475,10 +2492,13 @@ static void igt_atomic_prepare_crtc_commit(igt_pipe_t *pipe_obj, drmModeAtomicRe
 		igt_atomic_populate_crtc_req(req, pipe_obj, IGT_CRTC_ACTIVE, !!output);
 	}
 
-	pipe_obj->out_fence_fd = -1;
+	if (pipe_obj->out_fence_fd != -1) {
+		close(pipe_obj->out_fence_fd);
+		pipe_obj->out_fence_fd = -1;
+	}
+
 	if (pipe_obj->out_fence_requested)
 	{
-		pipe_obj->out_fence_requested = false;
 		igt_atomic_populate_crtc_req(req, pipe_obj, IGT_CRTC_OUT_FENCE_PTR,
 		    (uint64_t)(uintptr_t) &pipe_obj->out_fence_fd);
 	}
@@ -2559,27 +2579,6 @@ static int igt_atomic_commit(igt_display_t *display, uint32_t flags, void *user_
 	}
 
 	ret = drmModeAtomicCommit(display->drm_fd, req, flags, user_data);
-	if (!ret) {
-
-		for_each_pipe(display, pipe) {
-			igt_pipe_t *pipe_obj = &display->pipes[pipe];
-			igt_plane_t *plane;
-
-			/* reset fence_fd to prevent it from being set for the next commit */
-			for_each_plane_on_pipe(display, pipe, plane) {
-				igt_plane_set_fence_fd(plane, -1);
-			}
-
-			if (pipe_obj->out_fence_fd == -1)
-				continue;
-
-			igt_assert(pipe_obj->out_fence_fd >= 0);
-			ret = sync_fence_wait(pipe_obj->out_fence_fd, 1000);
-			igt_assert(ret == 0);
-			close(pipe_obj->out_fence_fd);
-			pipe_obj->out_fence_fd = -1;
-		}
-	}
 
 	drmModeAtomicFree(req);
 	return ret;
@@ -2602,6 +2601,11 @@ display_commit_changed(igt_display_t *display, enum igt_commit_style s)
 		if (s != COMMIT_UNIVERSAL)
 			pipe_obj->mode_changed = false;
 
+		if (s == COMMIT_ATOMIC && pipe_obj->out_fence_requested) {
+			pipe_obj->out_fence_requested = false;
+			igt_assert(pipe_obj->out_fence_fd >= 0);
+		}
+
 		for_each_plane_on_pipe(display, pipe, plane) {
 			plane->fb_changed = false;
 			plane->position_changed = false;
@@ -2611,6 +2615,10 @@ display_commit_changed(igt_display_t *display, enum igt_commit_style s)
 			    !(plane->type == DRM_PLANE_TYPE_PRIMARY ||
 			      plane->type == DRM_PLANE_TYPE_CURSOR))
 				plane->rotation_changed = false;
+
+			if (s == COMMIT_ATOMIC)
+				/* reset fence_fd to prevent it from being set for the next commit */
+				igt_plane_set_fence_fd(plane, -1);
 		}
 	}
 
@@ -2646,8 +2654,6 @@ static int do_display_commit(igt_display_t *display,
 
 	if (s == COMMIT_ATOMIC) {
 		ret = igt_atomic_commit(display, DRM_MODE_ATOMIC_ALLOW_MODESET, NULL);
-
-		CHECK_RETURN(ret, fail_on_error);
 	} else {
 		int valid_outs = 0;
 
@@ -2659,11 +2665,10 @@ static int do_display_commit(igt_display_t *display,
 				valid_outs++;
 
 			ret = igt_pipe_commit(pipe_obj, s, fail_on_error);
-			CHECK_RETURN(ret, fail_on_error);
+			if (ret)
+				break;
 		}
 
-		CHECK_RETURN(ret, fail_on_error);
-
 		if (valid_outs == 0) {
 			LOG_UNINDENT(display);
 
@@ -2671,10 +2676,9 @@ static int do_display_commit(igt_display_t *display,
 		}
 	}
 
-	LOG_UNINDENT(display);
 
-	if (ret)
-		return ret;
+	LOG_UNINDENT(display);
+	CHECK_RETURN(ret, fail_on_error);
 
 	display_commit_changed(display, s);
 
@@ -2821,7 +2825,10 @@ const char *igt_output_name(igt_output_t *output)
 
 drmModeModeInfo *igt_output_get_mode(igt_output_t *output)
 {
-	return &output->config.default_mode;
+	if (output->use_override_mode)
+		return &output->override_mode;
+	else
+		return &output->config.default_mode;
 }
 
 /**
@@ -2839,10 +2846,6 @@ void igt_output_override_mode(igt_output_t *output, drmModeModeInfo *mode)
 
 	if (mode)
 		output->override_mode = *mode;
-	else /* restore default_mode, may have been overwritten in igt_output_refresh */
-		kmstest_get_connector_default_mode(output->display->drm_fd,
-						   output->config.connector,
-						   &output->config.default_mode);
 
 	output->use_override_mode = !!mode;
 
@@ -2857,22 +2860,18 @@ void igt_output_set_pipe(igt_output_t *output, enum pipe pipe)
 
 	igt_assert(output->name);
 
-	if (output->pending_crtc_idx_mask) {
+	if (output->pending_pipe != PIPE_NONE) {
 		old_pipe = igt_output_get_driving_pipe(output);
 
 		old_pipe->mode_changed = true;
 	}
 
-	if (pipe == PIPE_NONE) {
-		LOG(display, "%s: set_pipe(any)\n", igt_output_name(output));
-		output->pending_crtc_idx_mask = 0;
-	} else {
-		LOG(display, "%s: set_pipe(%s)\n", igt_output_name(output),
-		    kmstest_pipe_name(pipe));
-		output->pending_crtc_idx_mask = 1 << pipe;
+	LOG(display, "%s: set_pipe(%s)\n", igt_output_name(output),
+	    kmstest_pipe_name(pipe));
+	output->pending_pipe = pipe;
 
+	if (pipe != PIPE_NONE)
 		display->pipes[pipe].mode_changed = true;
-	}
 
 	output->config.pipe_changed = true;
 
