@@ -31,6 +31,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stddef.h>
+#include <assert.h>
 
 #include <xf86drmMode.h>
 
@@ -46,7 +47,8 @@
  * @PIPE_A: First crtc.
  * @PIPE_B: Second crtc.
  * @PIPE_C: Third crtc.
- * @I915_MAX_PIPES: Max number of pipes allowed.
+ * ... and so on.
+ * @IGT_MAX_PIPES: Max number of pipes allowed.
  */
 enum pipe {
         PIPE_NONE = -1,
@@ -54,7 +56,10 @@ enum pipe {
         PIPE_A = 0,
         PIPE_B,
         PIPE_C,
-        I915_MAX_PIPES
+        PIPE_D,
+        PIPE_E,
+        PIPE_F,
+        IGT_MAX_PIPES
 };
 const char *kmstest_pipe_name(enum pipe pipe);
 int kmstest_pipe_to_index(char pipe);
@@ -109,6 +114,7 @@ extern const char *igt_crtc_prop_names[];
 enum igt_atomic_connector_properties {
        IGT_CONNECTOR_SCALING_MODE = 0,
        IGT_CONNECTOR_CRTC_ID,
+       IGT_CONNECTOR_DPMS,
        IGT_NUM_CONNECTOR_PROPS
 };
 
@@ -319,7 +325,6 @@ typedef struct {
 struct igt_pipe {
 	igt_display_t *display;
 	enum pipe pipe;
-	bool enabled;
 
 	int n_planes;
 	int plane_cursor;
@@ -356,7 +361,7 @@ typedef struct {
 	struct kmstest_connector_config config;
 	char *name;
 	bool force_reprobe;
-	unsigned long pending_crtc_idx_mask;
+	enum pipe pending_pipe;
 	bool use_override_mode;
 	drmModeModeInfo override_mode;
 } igt_output_t;
@@ -380,6 +385,8 @@ int  igt_display_try_commit_atomic(igt_display_t *display, uint32_t flags, void
 void igt_display_commit_atomic(igt_display_t *display, uint32_t flags, void *user_data);
 int  igt_display_try_commit2(igt_display_t *display, enum igt_commit_style s);
 int  igt_display_get_n_pipes(igt_display_t *display);
+void igt_display_require_output(igt_display_t *display);
+void igt_display_require_output_on_pipe(igt_display_t *display, enum pipe pipe);
 
 const char *igt_output_name(igt_output_t *output);
 drmModeModeInfo *igt_output_get_mode(igt_output_t *output);
@@ -451,19 +458,35 @@ static inline bool igt_output_is_connected(igt_output_t *output)
  * This for loop iterates over all outputs.
  */
 #define for_each_connected_output(display, output)		\
-	for (int i__ = 0;  i__ < (display)->n_outputs; i__++)	\
+	for (int i__ = 0;  assert(igt_can_fail()), i__ < (display)->n_outputs; i__++)	\
 		for_each_if (((output = &(display)->outputs[i__]), \
 			      igt_output_is_connected(output)))
 
+/**
+ * for_each_pipe:
+ * @pipe: The pipe to iterate.
+ *
+ * This for loop iterates over all pipes supported by IGT libraries.
+ *
+ * This should be used to enumerate per-pipe subtests since it has no runtime
+ * depencies.
+ */
+#define for_each_pipe_static(pipe) \
+	for (pipe = 0; pipe < IGT_MAX_PIPES; pipe++)
+
 /**
  * for_each_pipe:
  * @display: a pointer to an #igt_display_t structure
  * @pipe: The pipe to iterate.
  *
  * This for loop iterates over all pipes.
+ *
+ * Note that this cannot be used to enumerate per-pipe subtest names since it
+ * depends upon runtime probing of the actual kms driver that is being tested.
+ * Used #for_each_pipe_static instead.
  */
 #define for_each_pipe(display, pipe)					\
-	for (pipe = 0; pipe < igt_display_get_n_pipes(display); pipe++)
+	for (pipe = 0; assert(igt_can_fail()), pipe < igt_display_get_n_pipes(display); pipe++)
 
 /**
  * for_each_pipe_with_valid_output:
@@ -476,7 +499,7 @@ static inline bool igt_output_is_connected(igt_output_t *output)
  */
 #define for_each_pipe_with_valid_output(display, pipe, output) \
 	for (int con__ = pipe = 0; \
-	     pipe < igt_display_get_n_pipes((display)) && con__ < (display)->n_outputs; \
+	     assert(igt_can_fail()), pipe < igt_display_get_n_pipes((display)) && con__ < (display)->n_outputs; \
 	     con__ = (con__ + 1 < (display)->n_outputs) ? con__ + 1 : (pipe = pipe + 1, 0)) \
 		for_each_if (((output = &(display)->outputs[con__]), \
 			     igt_pipe_connector_valid(pipe, output)))
@@ -496,7 +519,7 @@ static inline bool igt_output_is_connected(igt_output_t *output)
 		for_each_if (igt_pipe_connector_valid(pipe, output))
 
 #define for_each_plane_on_pipe(display, pipe, plane)			\
-	for (int j__ = 0; (plane) = &(display)->pipes[(pipe)].planes[j__], \
+	for (int j__ = 0; assert(igt_can_fail()), (plane) = &(display)->pipes[(pipe)].planes[j__], \
 		     j__ < (display)->pipes[(pipe)].n_planes; j__++)
 
 #define IGT_FIXED(i,f)	((i) << 16 | (f))
@@ -533,6 +556,27 @@ static inline bool igt_output_is_connected(igt_output_t *output)
 	igt_assert_lt(0, drmModeAtomicAddProperty(req, output->config.connector->connector_id,\
 						  output->config.atomic_props_connector[prop], value))
 
+/*
+ * igt_pipe_refresh:
+ * @display: a pointer to an #igt_display_t structure
+ * @pipe: Pipe to refresh
+ * @force: Should be set to true if mode_blob is no longer considered
+ * to be valid, for example after doing an atomic commit during fork or closing display fd.
+ *
+ * Requests the pipe to be part of the state on next update.
+ * This is useful when state may have been out of sync after
+ * a fork, or we just want to be sure the pipe is included
+ * in the next commit.
+ */
+static inline void
+igt_pipe_refresh(igt_display_t *display, enum pipe pipe, bool force)
+{
+	if (force)
+		display->pipes[pipe].mode_blob = 0;
+
+	display->pipes[pipe].mode_changed = true;
+}
+
 void igt_enable_connectors(void);
 void igt_reset_connectors(void);
 
