@@ -553,7 +553,7 @@ fill_object(struct drm_i915_gem_exec_object2 *obj, uint32_t gem_handle,
 	memset(obj, 0, sizeof(*obj));
 	obj->handle = gem_handle;
 	obj->relocation_count = count;
-	obj->relocs_ptr = (uintptr_t)relocs;
+	obj->relocs_ptr = to_user_pointer(relocs);
 }
 
 static void exec_blit(int fd,
@@ -562,7 +562,7 @@ static void exec_blit(int fd,
 {
 	struct drm_i915_gem_execbuffer2 exec;
 
-	exec.buffers_ptr = (uintptr_t)objs;
+	exec.buffers_ptr = to_user_pointer(objs);
 	exec.buffer_count = count;
 	exec.batch_start_offset = 0;
 	exec.batch_len = batch_len * 4;
@@ -741,7 +741,7 @@ igt_render_copyfunc_t igt_get_render_copyfunc(int devid)
 		copy = gen7_render_copyfunc;
 	else if (IS_GEN8(devid))
 		copy = gen8_render_copyfunc;
-	else if (IS_GEN9(devid))
+	else if (IS_GEN9(devid) || IS_GEN10(devid))
 		copy = gen9_render_copyfunc;
 
 	return copy;
@@ -789,7 +789,7 @@ igt_fillfunc_t igt_get_gpgpu_fillfunc(int devid)
 		fill = gen7_gpgpu_fillfunc;
 	else if (IS_BROADWELL(devid))
 		fill = gen8_gpgpu_fillfunc;
-	else if (IS_GEN9(devid))
+	else if (IS_GEN9(devid) || IS_GEN10(devid))
 		fill = gen9_gpgpu_fillfunc;
 
 	return fill;
