@@ -26,9 +26,7 @@
  *
  */
 
-#ifndef ANDROID
-#define _GNU_SOURCE
-#else
+#ifdef HAVE_LIBGEN_H
 #include <libgen.h>
 #endif
 #include <stdio.h>
@@ -62,7 +60,7 @@
 
 #define VG(x) x
 #else
-#define VG(x)
+#define VG(x) do {} while (0)
 #endif
 
 #include "ioctl_wrappers.h"
@@ -553,22 +551,20 @@ uint32_t gem_create_stolen(int fd, uint64_t size)
 	return create.handle;
 }
 
-
-uint32_t __gem_create(int fd, int size)
+int __gem_create(int fd, uint64_t size, uint32_t *handle)
 {
-	struct drm_i915_gem_create create;
-	int ret;
+	struct drm_i915_gem_create create = {
+		.size = size,
+	};
+	int err = 0;
 
-	memset(&create, 0, sizeof(create));
-	create.handle = 0;
-	create.size = size;
-	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
-
-	if (ret < 0)
-		return 0;
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create) == 0)
+		*handle = create.handle;
+	else
+		err = -errno;
 
 	errno = 0;
-	return create.handle;
+	return err;
 }
 
 /**
@@ -583,15 +579,11 @@ uint32_t __gem_create(int fd, int size)
  */
 uint32_t gem_create(int fd, uint64_t size)
 {
-	struct drm_i915_gem_create create;
+	uint32_t handle;
 
-	memset(&create, 0, sizeof(create));
-	create.handle = 0;
-	create.size = size;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
-	igt_assert(create.handle);
+	igt_assert_eq(__gem_create(fd, size, &handle), 0);
 
-	return create.handle;
+	return handle;
 }
 
 /**
@@ -881,7 +873,7 @@ void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, uns
  * @handle: gem buffer object handle
  * @state: desired madvise state
  *
- * This is a wraps the MADVISE ioctl, which is used in libdrm to implement
+ * This wraps the MADVISE ioctl, which is used in libdrm to implement
  * opportunistic buffer object caching. Objects in the cache are set to DONTNEED
  * (internally in the kernel tracked as purgeable objects). When such a cached
  * object is in need again it must be set back to WILLNEED before first use.
@@ -906,12 +898,11 @@ int gem_madvise(int fd, uint32_t handle, int state)
  * gem_context_create:
  * @fd: open i915 drm file descriptor
  *
- * This is a wraps the CONTEXT_CREATE ioctl, which is used to allocate a new
- * hardware context. Not that similarly to gem_set_caching() this wrapper calls
- * igt_require() internally to correctly skip on kernels and platforms where hw
- * context support is not available.
+ * This wraps the CONTEXT_CREATE ioctl, which is used to allocate a new
+ * context. Note that similarly to gem_set_caching() this wrapper skips on
+ * kernels and platforms where context support is not available.
  *
- * Returns: The id of the allocated hw context.
+ * Returns: The id of the allocated context.
  */
 uint32_t gem_context_create(int fd)
 {
@@ -946,10 +937,9 @@ int __gem_context_destroy(int fd, uint32_t ctx_id)
 /**
  * gem_context_destroy:
  * @fd: open i915 drm file descriptor
- * @ctx_id: i915 hw context id
+ * @ctx_id: i915 context id
  *
- * This is a wraps the CONTEXT_DESTROY ioctl, which is used to free a hardware
- * context.
+ * This wraps the CONTEXT_DESTROY ioctl, which is used to free a context.
  */
 void gem_context_destroy(int fd, uint32_t ctx_id)
 {
@@ -975,12 +965,10 @@ int __gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
 /**
  * gem_context_get_param:
  * @fd: open i915 drm file descriptor
- * @p: i915 hw context parameter
+ * @p: i915 context parameter
  *
- * This is a wraps the CONTEXT_GET_PARAM ioctl, which is used to free a hardware
- * context. Not that similarly to gem_set_caching() this wrapper calls
- * igt_require() internally to correctly skip on kernels and platforms where hw
- * context parameter support is not available.
+ * This wraps the CONTEXT_GET_PARAM ioctl, which is used to get a context
+ * parameter.
  */
 void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
 {
@@ -1001,12 +989,10 @@ int __gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 /**
  * gem_context_set_param:
  * @fd: open i915 drm file descriptor
- * @p: i915 hw context parameter
+ * @p: i915 context parameter
  *
- * This is a wraps the CONTEXT_SET_PARAM ioctl, which is used to free a hardware
- * context. Not that similarly to gem_set_caching() this wrapper calls
- * igt_require() internally to correctly skip on kernels and platforms where hw
- * context parameter support is not available.
+ * This wraps the CONTEXT_SET_PARAM ioctl, which is used to set a context
+ * parameter.
  */
 void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 {
@@ -1016,9 +1002,9 @@ void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 /**
  * gem_context_require_param:
  * @fd: open i915 drm file descriptor
- * @param: i915 hw context parameter
+ * @param: i915 context parameter
  *
- * Feature test macro to query whether hw context parameter support for @param
+ * Feature test macro to query whether context parameter support for @param
  * is available. Automatically skips through igt_require() if not.
  */
 void gem_context_require_param(int fd, uint64_t param)
@@ -1102,7 +1088,7 @@ void gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uin
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  *
- * This is a wraps the SW_FINISH ioctl, which is used to flush out frontbuffer
+ * This wraps the SW_FINISH ioctl, which is used to flush out frontbuffer
  * rendering done through the direct cpu memory mappings. Shipping userspace
  * does _not_ call this after frontbuffer rendering through gtt memory mappings.
  */
@@ -1121,7 +1107,7 @@ void gem_sw_finish(int fd, uint32_t handle)
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  *
- * This is a wraps the BUSY ioctl, which tells whether a buffer object is still
+ * This wraps the BUSY ioctl, which tells whether a buffer object is still
  * actively used by the gpu in a execbuffer.
  *
  * Returns: The busy state of the buffer object.
@@ -1198,6 +1184,57 @@ bool gem_uses_full_ppgtt(int fd)
 	return gem_gtt_type(fd) > 1;
 }
 
+/**
+ * gem_gpu_reset_type:
+ * @fd: open i915 drm file descriptor
+ *
+ * Query whether reset-engine (2), global-reset (1) or reset-disable (0)
+ * is available.
+ *
+ * Returns: GPU reset type available
+ */
+int gem_gpu_reset_type(int fd)
+{
+	struct drm_i915_getparam gp;
+	int gpu_reset_type = -1;
+
+	memset(&gp, 0, sizeof(gp));
+	gp.param = I915_PARAM_HAS_GPU_RESET;
+	gp.value = &gpu_reset_type;
+	drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+
+	return gpu_reset_type;
+}
+
+/**
+ * gem_gpu_reset_enabled:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses hangchecks
+ * and can reset the GPU upon hang detection. Note that this is also true when
+ * reset-engine (the lightweight, single engine reset) is available.
+ *
+ * Returns: Whether the driver will detect hangs and perform a reset.
+ */
+bool gem_gpu_reset_enabled(int fd)
+{
+	return gem_gpu_reset_type(fd) > 0;
+}
+
+/**
+ * gem_engine_reset_enabled:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to check whether the kernel internally uses hangchecks
+ * and can reset individual engines upon hang detection.
+ *
+ * Returns: Whether the driver will detect hangs and perform an engine reset.
+ */
+bool gem_engine_reset_enabled(int fd)
+{
+	return gem_gpu_reset_type(fd) > 1;
+}
+
 /**
  * gem_available_fences:
  * @fd: open i915 drm file descriptor
@@ -1553,13 +1590,23 @@ void gem_require_caching(int fd)
 
 void igt_require_gem(int fd)
 {
+	char path[256];
 	int err;
 
 	igt_require_intel(fd);
 
+	/* We only want to use the throttle-ioctl for its -EIO reporting
+	 * of a wedged device, not for actually waiting on outstanding
+	 * requests! So create a new drm_file for the device that is clean.
+	 */
+	snprintf(path, sizeof(path), "/proc/self/fd/%d", fd);
+	fd = open(path, O_RDWR);
+	igt_assert_lte(0, fd);
+
 	err = 0;
 	if (ioctl(fd, DRM_IOCTL_I915_GEM_THROTTLE))
 		err = -errno;
+	close(fd);
 
 	igt_require_f(err == 0, "Unresponsive i915/GEM device\n");
 }
