@@ -74,7 +74,7 @@ static void copy(int fd, uint32_t batch, uint32_t src, uint32_t dst)
 	gem_reloc[0].target_handle = dst;
 	gem_reloc[0].read_domains = I915_GEM_DOMAIN_RENDER;
 	gem_reloc[0].write_domain = I915_GEM_DOMAIN_RENDER;
-	gem_reloc[0].presumed_offset = 0;
+	gem_reloc[0].presumed_offset = -1;
 
 	gem_reloc[1].offset = 7 * sizeof(uint32_t);
 	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
@@ -83,17 +83,17 @@ static void copy(int fd, uint32_t batch, uint32_t src, uint32_t dst)
 	gem_reloc[1].target_handle = src;
 	gem_reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	gem_reloc[1].write_domain = 0;
-	gem_reloc[1].presumed_offset = 0;
+	gem_reloc[1].presumed_offset = -1;
 
 	memset(gem_exec, 0, sizeof(gem_exec));
 	gem_exec[0].handle = src;
 	gem_exec[1].handle = dst;
 	gem_exec[2].handle = batch;
 	gem_exec[2].relocation_count = 2;
-	gem_exec[2].relocs_ptr = (uintptr_t)gem_reloc;
+	gem_exec[2].relocs_ptr = to_user_pointer(gem_reloc);
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 3;
 	execbuf.batch_len = 4096;
 	execbuf.flags = use_blt;
@@ -110,7 +110,7 @@ static void exec(int fd, uint32_t handle)
 	gem_exec.handle = handle;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(&gem_exec);
 	execbuf.buffer_count = 1;
 	execbuf.batch_len = 4096;
 
@@ -247,6 +247,7 @@ igt_main
 
 	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 	}
 
 	igt_subtest("basic") {
