@@ -134,8 +134,8 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 			igt_assert(flags & COHERENT);
 			map = gem_mmap__wc(fd, obj[0].handle, 0, 4096, PROT_WRITE);
 			gem_set_domain(fd, obj[0].handle,
-				       I915_GEM_DOMAIN_GTT,
-				       I915_GEM_DOMAIN_GTT);
+				       I915_GEM_DOMAIN_WC,
+				       I915_GEM_DOMAIN_WC);
 		} else {
 			snoop = flags & COHERENT;
 			gem_set_caching(fd, obj[0].handle, snoop);
@@ -149,8 +149,8 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 			map[i] = 0xabcdabcd;
 
 		gem_set_domain(fd, obj[0].handle,
-			       I915_GEM_DOMAIN_GTT,
-			       I915_GEM_DOMAIN_GTT);
+			       I915_GEM_DOMAIN_WC,
+			       I915_GEM_DOMAIN_WC);
 
 		/* Prepara a mappable binding to prevent pread mighrating */
 		if (!snoop) {
@@ -160,7 +160,7 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 		}
 
 		memset(&execbuf, 0, sizeof(execbuf));
-		execbuf.buffers_ptr = (uintptr_t)obj;
+		execbuf.buffers_ptr = to_user_pointer(obj);
 		execbuf.buffer_count = 3;
 		execbuf.flags = ring | (1 << 11) | (1<<12);
 		if (gen < 6)
@@ -177,7 +177,7 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 		ptr = gem_mmap__wc(fd, obj[1].handle, 0, 64*1024,
 				   PROT_WRITE | PROT_READ);
 		gem_set_domain(fd, obj[1].handle,
-			       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+			       I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
 
 		memset(reloc0, 0, sizeof(reloc0));
 		for (i = 0; i < 1024; i++) {
@@ -211,7 +211,7 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 		ptr = gem_mmap__wc(fd, obj[2].handle, 0, 64*1024,
 				   PROT_WRITE | PROT_READ);
 		gem_set_domain(fd, obj[2].handle,
-			       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+			       I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
 
 		memset(reloc1, 0, sizeof(reloc1));
 		for (i = 0; i < 1024; i++) {
@@ -248,8 +248,8 @@ static void run(int fd, unsigned ring, int nchild, int timeout,
 
 			/* Inspect a different cacheline each iteration */
 			i = 16 * (idx % 64) + (idx / 64);
-			obj[1].relocs_ptr = (uintptr_t)&reloc0[i];
-			obj[2].relocs_ptr = (uintptr_t)&reloc1[i];
+			obj[1].relocs_ptr = to_user_pointer(&reloc0[i]);
+			obj[2].relocs_ptr = to_user_pointer(&reloc1[i]);
 			igt_assert_eq_u64(reloc0[i].presumed_offset, obj[0].offset);
 			igt_assert_eq_u64(reloc1[i].presumed_offset, obj[0].offset);
 			execbuf.batch_start_offset =  64*i;
@@ -263,7 +263,7 @@ overwrite:
 			gem_execbuf(fd, &execbuf);
 
 			if (flags & SET_DOMAIN) {
-				unsigned domain = flags & WC ? I915_GEM_DOMAIN_GTT : I915_GEM_DOMAIN_CPU;
+				unsigned domain = flags & WC ? I915_GEM_DOMAIN_WC : I915_GEM_DOMAIN_CPU;
 				igt_while_interruptible(flags & INTERRUPTIBLE)
 					gem_set_domain(fd, obj[0].handle,
 						       domain, (flags & WRITE) ? domain : 0);
@@ -392,7 +392,7 @@ static void batch(int fd, unsigned ring, int nchild, int timeout,
 			map[i] = 0xabcdabcd;
 
 		memset(&execbuf, 0, sizeof(execbuf));
-		execbuf.buffers_ptr = (uintptr_t)obj;
+		execbuf.buffers_ptr = to_user_pointer(obj);
 		execbuf.buffer_count = 2;
 		execbuf.flags = ring | (1 << 11) | (1<<12);
 		if (gen < 6)
@@ -403,7 +403,7 @@ static void batch(int fd, unsigned ring, int nchild, int timeout,
 		igt_require(__gem_execbuf(fd, &execbuf) == 0);
 
 		obj[1].relocation_count = 1;
-		obj[1].relocs_ptr = (uintptr_t)&reloc;
+		obj[1].relocs_ptr = to_user_pointer(&reloc);
 
 		switch (mode) {
 		case BATCH_CPU:
@@ -450,6 +450,10 @@ static void batch(int fd, unsigned ring, int nchild, int timeout,
 					break;
 
 				case BATCH_WC:
+					gem_set_domain(fd, obj[1].handle,
+						       I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
+					break;
+
 				case BATCH_GTT:
 					gem_set_domain(fd, obj[1].handle,
 						       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
@@ -570,7 +574,6 @@ igt_main
 		{ NULL }
 	};
 	unsigned cpu = x86_64_features();
-	int gen = -1;
 	int fd = -1;
 
 	igt_skip_on_simulation();
@@ -578,8 +581,9 @@ igt_main
 	igt_fixture {
 		igt_require(igt_setup_clflush());
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 		gem_require_mmap_wc(fd);
-		gen = intel_gen(intel_get_drm_devid(fd));
+		igt_require(gem_can_store_dword(fd, 0));
 		igt_info("Has LLC? %s\n", yesno(gem_has_llc(fd)));
 
 		if (cpu) {
@@ -598,8 +602,7 @@ igt_main
 
 		igt_fixture {
 			gem_require_ring(fd, ring);
-			igt_skip_on_f(gen == 6 && e->exec_id == I915_EXEC_BSD,
-				      "MI_STORE_DATA broken on gen6 bsd\n");
+			igt_require(gem_can_store_dword(fd, ring));
 		}
 
 		for (const struct batch *b = batches; b->name; b++) {
