@@ -65,7 +65,7 @@ static void cursor_disable(data_t *data)
 	igt_output_t *output = data->output;
 	igt_plane_t *cursor;
 
-	cursor = igt_output_get_plane(output, IGT_PLANE_CURSOR);
+	cursor = igt_output_get_plane_type(output, DRM_PLANE_TYPE_CURSOR);
 	igt_plane_set_fb(cursor, NULL);
 }
 
@@ -226,7 +226,7 @@ static void test_edges(data_t *data, unsigned int edges)
 	}
 }
 
-static bool prepare_crtc(data_t *data)
+static void prepare_crtc(data_t *data)
 {
 	drmModeModeInfo *mode;
 	igt_display_t *display = &data->display;
@@ -235,13 +235,6 @@ static bool prepare_crtc(data_t *data)
 	/* select the pipe we want to use */
 	igt_output_set_pipe(data->output, data->pipe);
 	cursor_disable(data);
-	igt_display_commit(display);
-
-	if (!data->output->valid) {
-		igt_output_set_pipe(data->output, PIPE_ANY);
-		igt_display_commit(display);
-		return false;
-	}
 
 	mode = igt_output_get_mode(data->output);
 	igt_create_pattern_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
@@ -249,7 +242,7 @@ static bool prepare_crtc(data_t *data)
 			      LOCAL_DRM_FORMAT_MOD_NONE,
 			      &data->primary_fb);
 
-	primary = igt_output_get_plane(data->output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, &data->primary_fb);
 
 	igt_display_commit(display);
@@ -261,7 +254,7 @@ static bool prepare_crtc(data_t *data)
 	if (data->pipe_crc)
 		igt_pipe_crc_free(data->pipe_crc);
 
-	data->pipe_crc = igt_pipe_crc_new_nonblock(data->pipe,
+	data->pipe_crc = igt_pipe_crc_new_nonblock(data->drm_fd, data->pipe,
 						   INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	/* make sure cursor is disabled */
@@ -272,8 +265,6 @@ static bool prepare_crtc(data_t *data)
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
-
-	return true;
 }
 
 static void cleanup_crtc(data_t *data)
@@ -286,7 +277,7 @@ static void cleanup_crtc(data_t *data)
 
 	igt_remove_fb(data->drm_fd, &data->primary_fb);
 
-	primary = igt_output_get_plane(data->output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, NULL);
 
 	igt_output_set_pipe(data->output, PIPE_ANY);
@@ -300,9 +291,8 @@ static void test_crtc(data_t *data, unsigned int edges)
 
 	create_cursor_fb(data, data->curw, data->curh);
 
-	for_each_connected_output(display, data->output) {
-		if (!prepare_crtc(data))
-			continue;
+	for_each_valid_output_on_pipe(display, data->pipe, data->output) {
+		prepare_crtc(data);
 
 		valid_tests++;
 
@@ -384,7 +374,7 @@ int main(int argc, char **argv)
 
 		kmstest_set_vt_graphics_mode();
 
-		igt_require_pipe_crc();
+		igt_require_pipe_crc(data.drm_fd);
 
 		igt_display_init(&data.display, data.drm_fd);
 	}
