@@ -22,7 +22,6 @@
  *
  */
 
-#define _GNU_SOURCE
 #include <sched.h>
 #include <sys/poll.h>
 
@@ -47,14 +46,14 @@
 IGT_TEST_DESCRIPTION("Stress legacy cursor ioctl");
 
 static void stress(igt_display_t *display,
-		   int pipe, int num_children, unsigned mode,
+		   enum pipe pipe, int num_children, unsigned mode,
 		   int timeout)
 {
 	struct drm_mode_cursor arg;
 	uint64_t *results;
 	bool torture;
 	int n;
-	unsigned crtc_id[I915_MAX_PIPES], num_crtcs;
+	unsigned crtc_id[IGT_MAX_PIPES], num_crtcs;
 
 	torture = false;
 	if (num_children < 0) {
@@ -148,7 +147,7 @@ static void stress(igt_display_t *display,
 	munmap(results, 4096);
 }
 
-static igt_output_t *set_fb_on_crtc(igt_display_t *display, int pipe, struct igt_fb *fb_info)
+static igt_output_t *set_fb_on_crtc(igt_display_t *display, enum pipe pipe, struct igt_fb *fb_info)
 {
 	igt_output_t *output;
 
@@ -156,7 +155,7 @@ static igt_output_t *set_fb_on_crtc(igt_display_t *display, int pipe, struct igt
 		drmModeModeInfoPtr mode;
 		igt_plane_t *primary;
 
-		if (output->pending_crtc_idx_mask)
+		if (output->pending_pipe != PIPE_NONE)
 			continue;
 
 		igt_output_set_pipe(output, pipe);
@@ -464,7 +463,7 @@ static void flip(igt_display_t *display,
 		igt_remove_fb(display->drm_fd, &cursor_fb2);
 }
 
-static inline uint32_t pipe_select(int pipe)
+static inline uint32_t pipe_select(enum pipe pipe)
 {
 	if (pipe > 1)
 		return pipe << DRM_VBLANK_HIGH_CRTC_SHIFT;
@@ -474,7 +473,7 @@ static inline uint32_t pipe_select(int pipe)
 		return 0;
 }
 
-static unsigned get_vblank(int fd, int pipe, unsigned flags)
+static unsigned get_vblank(int fd, enum pipe pipe, unsigned flags)
 {
 	union drm_wait_vblank vbl;
 
@@ -491,92 +490,6 @@ enum basic_flip_cursor {
 	FLIP_AFTER_CURSOR
 };
 
-static uint32_t *make_busy(int fd, uint32_t target)
-{
-	const int gen = intel_gen(intel_get_drm_devid(fd));
-	struct drm_i915_gem_exec_object2 obj[2];
-	struct drm_i915_gem_relocation_entry reloc[2];
-	struct drm_i915_gem_execbuffer2 execbuf;
-	uint32_t *batch;
-	int i;
-
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)obj;
-	execbuf.buffer_count = 2;
-
-	memset(obj, 0, sizeof(obj));
-	obj[0].handle = target;
-	obj[1].handle = gem_create(fd, 4096);
-	batch = gem_mmap__wc(fd, obj[1].handle, 0, 4096, PROT_WRITE);
-	gem_set_domain(fd, obj[1].handle,
-			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
-
-
-	obj[1].relocs_ptr = (uintptr_t)reloc;
-	obj[1].relocation_count = 2;
-	memset(reloc, 0, sizeof(reloc));
-
-	reloc[0].target_handle = obj[1].handle; /* recurse */
-	reloc[0].presumed_offset = 0;
-	reloc[0].offset = sizeof(uint32_t);
-	reloc[0].delta = 0;
-	reloc[0].read_domains = I915_GEM_DOMAIN_COMMAND;
-	reloc[0].write_domain = 0;
-
-	reloc[1].target_handle = target;
-	reloc[1].presumed_offset = 0;
-	reloc[1].offset = 1024;
-	reloc[1].delta = 0;
-	reloc[1].read_domains = I915_GEM_DOMAIN_COMMAND;
-	reloc[1].write_domain = I915_GEM_DOMAIN_COMMAND;
-
-	i = 0;
-	batch[i] = MI_BATCH_BUFFER_START;
-	if (gen >= 8) {
-		batch[i] |= 1 << 8 | 1;
-		batch[++i] = 0;
-		batch[++i] = 0;
-	} else if (gen >= 6) {
-		batch[i] |= 1 << 8;
-		batch[++i] = 0;
-	} else {
-		batch[i] |= 2 << 6;
-		batch[++i] = 0;
-		if (gen < 4) {
-			batch[i] |= 1;
-			reloc[0].delta = 1;
-		}
-	}
-	i++;
-
-	gem_execbuf(fd, &execbuf);
-	gem_close(fd, obj[1].handle);
-
-	return batch;
-}
-
-static void cancel_busy(uint32_t *busy)
-{
-	*busy = MI_BATCH_BUFFER_END;
-	munmap(busy, 4096);
-}
-
-static uint32_t *
-make_fb_busy(int fd, const struct igt_fb *fb)
-{
-	uint32_t *busy;
-
-	busy = make_busy(fd, fb->gem_handle);
-	igt_assert(gem_bo_busy(fd, fb->gem_handle));
-
-	return busy;
-}
-
-static void finish_fb_busy(uint32_t *busy)
-{
-	cancel_busy(busy);
-}
-
 #define BASIC_BUSY 0x1
 
 static void basic_flip_cursor(igt_display_t *display,
@@ -589,7 +502,8 @@ static void basic_flip_cursor(igt_display_t *display,
 	struct igt_fb fb_info, cursor_fb, cursor_fb2, argb_fb;
 	unsigned vblank_start;
 	enum pipe pipe = find_connected_pipe(display, false);
-	uint32_t *busy;
+	igt_spin_t *spin;
+	int i, miss1 = 0, miss2 = 0, delta;
 
 	if (mode >= flip_test_atomic)
 		igt_require(display->is_atomic);
@@ -610,67 +524,89 @@ static void basic_flip_cursor(igt_display_t *display,
 	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
-	/* Bind the cursor first to warm up */
-	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-
-	/* Start with a synchronous query to align with the vblank */
-	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-
-	busy = NULL;
-	if (flags & BASIC_BUSY)
-		busy = make_fb_busy(display->drm_fd, &fb_info);
-
-	switch (order) {
-	case FLIP_BEFORE_CURSOR:
-		switch (mode) {
-		default:
-			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
-			break;
-		case flip_test_atomic_transitions:
-		case flip_test_atomic_transitions_varying_size:
-			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
-			break;
-		}
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	for (i = 0; i < 25; i++) {
+		bool miss;
 
+		/* Bind the cursor first to warm up */
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		break;
 
-	case FLIP_AFTER_CURSOR:
-		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		spin = NULL;
+		if (flags & BASIC_BUSY)
+			spin = igt_spin_batch_new(display->drm_fd, 0, 0, fb_info.gem_handle);
 
-		switch (mode) {
-		default:
-			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
-			break;
-		case flip_test_atomic_transitions:
-		case flip_test_atomic_transitions_varying_size:
-			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+		/* Start with a synchronous query to align with the vblank */
+		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+		switch (order) {
+		case FLIP_BEFORE_CURSOR:
+			switch (mode) {
+			default:
+				flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+				break;
+			case flip_test_atomic_transitions:
+			case flip_test_atomic_transitions_varying_size:
+				transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+				break;
+			}
+
+			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			miss = delta != 0;
+
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 			break;
+
+		case FLIP_AFTER_CURSOR:
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+
+			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			miss = delta != 0;
+
+			switch (mode) {
+			default:
+				flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+				break;
+			case flip_test_atomic_transitions:
+			case flip_test_atomic_transitions_varying_size:
+				transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+				break;
+			}
 		}
+
+		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+
+		if (spin) {
+			struct pollfd pfd = { display->drm_fd, POLLIN };
+			igt_assert(poll(&pfd, 1, 0) == 0);
+			igt_spin_batch_free(display->drm_fd, spin);
+		}
+
+		if (miss)
+			{ /* compare nothing, already failed */ }
+		else if (!cursor_slowpath(mode))
+			miss = delta != 0;
+		else
+			miss = delta != 0 && delta != 1;
+
+		miss1 += miss;
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+
+		if (miss1)
+			continue;
+
+		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+
+		if (!mode_requires_extra_vblank(mode))
+			miss2 += delta != 1;
+		else
+			miss2 += delta != 1 && delta != 2;
 	}
 
-	if (!cursor_slowpath(mode))
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-	else
-		igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
-
-	if (busy) {
-		struct pollfd pfd = { display->drm_fd, POLLIN };
-		igt_assert(poll(&pfd, 1, 0) == 0);
-		finish_fb_busy(busy);
-	}
-
-	igt_set_timeout(1, "Stuck page flip");
-	igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
-
-	if (!mode_requires_extra_vblank(mode))
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
-	else
-		igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 2);
-
-	igt_reset_timeout();
+	igt_fail_on_f(miss1 > 2 || miss1 + miss2 > 5, "Failed to evade %i vblanks and missed %i page flips\n", miss1, miss2);
+	if (miss1 || miss2)
+		igt_info("Failed to evade %i vblanks and missed %i page flips\n", miss1, miss2);
 
 	do_cleanup_display(display);
 	igt_remove_fb(display->drm_fd, &fb_info);
@@ -848,7 +784,7 @@ static bool skip_on_unsupported_nonblocking_modeset(igt_display_t *display)
 
 	/* Force the next state to update all crtc's, to synchronize with the nonblocking modeset. */
 	for_each_pipe(display, pipe)
-		display->pipes[pipe].mode_changed = true;
+		igt_pipe_refresh(display, pipe, false);
 
 	return false;
 }
@@ -1035,8 +971,8 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 
 		if (modeset) {
 			/* wait for pending modeset and page flip to complete, to prevent -EBUSY */
-			display->pipes[pipe].mode_changed = true;
-			display->pipes[pipe2].mode_changed = true;
+			igt_pipe_refresh(display, pipe, false);
+			igt_pipe_refresh(display, pipe2, false);
 			igt_display_commit2(display, COMMIT_ATOMIC);
 		}
 	}
@@ -1264,7 +1200,7 @@ static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
 
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+	pipe_crc = igt_pipe_crc_new(display->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
@@ -1312,7 +1248,7 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	igt_pipe_crc_t *pipe_crc;
 	igt_pipe_t *pipe_connected = &display->pipes[pipe];
 	igt_plane_t *plane_primary = igt_pipe_get_plane_type(pipe_connected, DRM_PLANE_TYPE_PRIMARY);
-	igt_crc_t crcs[3];
+	igt_crc_t crcs[2];
 
 	if (atomic)
 		igt_require(display->is_atomic);
@@ -1326,7 +1262,7 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+	pipe_crc = igt_pipe_crc_new_nonblock(display->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
@@ -1349,11 +1285,21 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	igt_plane_set_fb(plane_primary, &fb_info[0]);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
+	/*
+	 * We must enable CRC collecting here since this may force
+	 * a modeset, and this loop is timing sensitive.
+	 */
+	igt_pipe_crc_start(pipe_crc);
+
 	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
 	for (int i = 1; i >= 0; i--) {
-		uint32_t *busy;
+		igt_spin_t *spin;
+		igt_crc_t *received_crcs = NULL;
+		int ncrcs;
+		static const int max_crcs = 8;
 
-		busy = make_fb_busy(display->drm_fd, &fb_info[1]);
+		spin = igt_spin_batch_new(display->drm_fd, 0, 0,
+					  fb_info[1].gem_handle);
 
 		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
@@ -1362,9 +1308,9 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 
 		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
-		igt_pipe_crc_collect_crc(pipe_crc, &crcs[2]);
+		ncrcs = igt_pipe_crc_get_crcs(pipe_crc, max_crcs, &received_crcs);
 
-		finish_fb_busy(busy);
+		igt_spin_batch_free(display->drm_fd, spin);
 
 		igt_set_timeout(1, "Stuck page flip");
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
@@ -1375,13 +1321,28 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 		igt_plane_set_fb(plane_primary, &fb_info[0]);
 		igt_display_commit2(display, COMMIT_UNIVERSAL);
 
-		igt_assert_crc_equal(&crcs[i], &crcs[2]);
+		/*
+		 * We collect the crc nonblockingly, and should have at least 1
+		 * but not so many crcs that we overflow. Last CRC is the only
+		 * one we care about here. Other CRCs may have been from before
+		 * the cursor update and can contain garbage.
+		 */
+		igt_assert(ncrcs > 0 && ncrcs < max_crcs);
+
+		igt_assert_crc_equal(&crcs[i], &received_crcs[ncrcs - 1]);
+		free(received_crcs);
 	}
 
 	do_cleanup_display(display);
 	igt_remove_fb(display->drm_fd, &fb_info[1]);
 	igt_remove_fb(display->drm_fd, &fb_info[0]);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
+
+	/*
+	 * igt_pipe_crc_stop() may force a modeset for workarounds, call
+	 * it after do_cleanup_display since we disable the display anyway.
+	 */
+	igt_pipe_crc_stop(pipe_crc);
 	igt_pipe_crc_free(pipe_crc);
 }
 
@@ -1402,7 +1363,8 @@ igt_main
 	}
 
 	igt_subtest_group {
-		for (int n = 0; n < I915_MAX_PIPES; n++) {
+		enum pipe n;
+		for_each_pipe_static(n) {
 			errno = 0;
 
 			igt_fixture {
@@ -1512,10 +1474,13 @@ igt_main
 		igt_subtest_f("%sflip-before-cursor-%s", prefix, modes[i])
 			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR, 0);
 
-		if (!cursor_slowpath(i))
-			igt_subtest_f("%sbusy-flip-before-cursor-%s", prefix, modes[i])
+		if (!cursor_slowpath(i)) {
+			igt_subtest_f("%sbusy-flip-before-cursor-%s", prefix, modes[i]) {
+				igt_require_gem(display.drm_fd);
 				basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR,
 						  BASIC_BUSY);
+			}
+		}
 
 		igt_subtest_f("%sflip-after-cursor-%s", prefix, modes[i])
 			basic_flip_cursor(&display, i, FLIP_AFTER_CURSOR, 0);
