@@ -207,8 +207,9 @@ test_plane_position_with_output(data_t *data,
 
 	if (flags & TEST_POSITION_FULLY_COVERED)
 		igt_assert_crc_equal(&test.reference_crc, &crc);
-	else
+	else {
 		;/* FIXME: missing reference CRCs */
+	}
 
 	igt_assert_crc_equal(&crc, &crc2);
 
@@ -354,9 +355,6 @@ test_plane_panning(data_t *data, enum pipe pipe, int plane,
 	igt_output_t *output;
 	int connected_outs = 0;
 
-	igt_skip_on(pipe >= data->display.n_pipes);
-	igt_skip_on(plane >= data->display.pipes[pipe].n_planes);
-
 	for_each_valid_output_on_pipe(&data->display, pipe, output) {
 		test_plane_panning_with_output(data, pipe, plane, output,
 						flags);
@@ -369,6 +367,11 @@ test_plane_panning(data_t *data, enum pipe pipe, int plane,
 static void
 run_tests_for_pipe_plane(data_t *data, enum pipe pipe)
 {
+	igt_fixture {
+		igt_skip_on(pipe >= data->display.n_pipes);
+		igt_require(data->display.pipes[pipe].n_planes > 0);
+	}
+
 	igt_subtest_f("plane-position-covered-pipe-%s-planes",
 		      kmstest_pipe_name(pipe)) {
 		int n_planes = data->display.pipes[pipe].n_planes;
@@ -422,6 +425,7 @@ static data_t data;
 
 igt_main
 {
+	enum pipe pipe;
 
 	igt_skip_on_simulation();
 
@@ -434,7 +438,7 @@ igt_main
 		igt_display_init(&data.display, data.drm_fd);
 	}
 
-	for (int pipe = 0; pipe < I915_MAX_PIPES; pipe++)
+	for_each_pipe_static(pipe)
 		run_tests_for_pipe_plane(&data, pipe);
 
 	igt_fixture {
