@@ -26,7 +26,6 @@
  *
  */
 
-#define _GNU_SOURCE
 #include "igt.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +39,8 @@
 
 #include "intel_bufmgr.h"
 
+IGT_TEST_DESCRIPTION("Render P-States tests - verify GPU frequency changes");
+
 static int drm_fd;
 
 static const char sysfs_base_path[] = "/sys/class/drm/card%d/gt_%s_freq_mhz";
@@ -50,17 +51,25 @@ enum {
 	RP0,
 	RP1,
 	RPn,
+	BOOST,
 	NUMFREQ
 };
 
 static int origfreqs[NUMFREQ];
 
-struct junk {
+struct sysfs_file {
 	const char *name;
 	const char *mode;
 	FILE *filp;
-} stuff[] = {
-	{ "cur", "r", NULL }, { "min", "rb+", NULL }, { "max", "rb+", NULL }, { "RP0", "r", NULL }, { "RP1", "r", NULL }, { "RPn", "r", NULL }, { NULL, NULL, NULL }
+} sysfs_files[] = {
+	{ "cur", "r", NULL },
+	{ "min", "rb+", NULL },
+	{ "max", "rb+", NULL },
+	{ "RP0", "r", NULL },
+	{ "RP1", "r", NULL },
+	{ "RPn", "r", NULL },
+	{ "boost", "rb+", NULL },
+	{ NULL, NULL, NULL }
 };
 
 static int readval(FILE *filp)
@@ -80,7 +89,7 @@ static void read_freqs(int *freqs)
 	int i;
 
 	for (i = 0; i < NUMFREQ; i++)
-		freqs[i] = readval(stuff[i].filp);
+		freqs[i] = readval(sysfs_files[i].filp);
 }
 
 static void nsleep(unsigned long ns)
@@ -130,7 +139,7 @@ static int do_writeval(FILE *filp, int val, int lerrno, bool readback_check)
 			igt_assert_eq(readval(filp), orig);
 	} else {
 		/* Expecting no error */
-		igt_assert_neq(ret, 0);
+		igt_assert_lt(0, ret);
 		wait_freq_settle();
 		if (readback_check)
 			igt_assert_eq(readval(filp), val);
@@ -142,7 +151,7 @@ static int do_writeval(FILE *filp, int val, int lerrno, bool readback_check)
 #define writeval_inval(filp, val) do_writeval(filp, val, EINVAL, true)
 #define writeval_nocheck(filp, val) do_writeval(filp, val, 0, false)
 
-static void checkit(const int *freqs)
+static void check_freq_constraints(const int *freqs)
 {
 	igt_assert_lte(freqs[MIN], freqs[MAX]);
 	igt_assert_lte(freqs[CUR], freqs[MAX]);
@@ -155,29 +164,19 @@ static void checkit(const int *freqs)
 	igt_assert_neq(freqs[RP1], 0);
 }
 
-static void matchit(const int *freqs1, const int *freqs2)
-{
-	igt_assert_eq(freqs1[CUR], freqs2[CUR]);
-	igt_assert_eq(freqs1[MIN], freqs2[MIN]);
-	igt_assert_eq(freqs1[MAX], freqs2[MAX]);
-	igt_assert_eq(freqs1[RP0], freqs2[RP0]);
-	igt_assert_eq(freqs1[RP1], freqs2[RP1]);
-	igt_assert_eq(freqs1[RPn], freqs2[RPn]);
-}
-
 static void dump(const int *freqs)
 {
 	int i;
 
 	igt_debug("gt freq (MHz):");
 	for (i = 0; i < NUMFREQ; i++)
-		igt_debug("  %s=%d", stuff[i].name, freqs[i]);
+		igt_debug("  %s=%d", sysfs_files[i].name, freqs[i]);
 
 	igt_debug("\n");
 }
 
 enum load {
-	LOW,
+	LOW = 0,
 	HIGH
 };
 
@@ -195,9 +194,10 @@ static struct load_helper {
 
 static void load_helper_signal_handler(int sig)
 {
-	if (sig == SIGUSR2)
-		lh.load = lh.load == LOW ? HIGH : LOW;
-	else
+	if (sig == SIGUSR2) {
+		lh.load = !lh.load;
+		igt_debug("Switching background load to %s\n", lh.load ? "high" : "low");
+	} else
 		lh.exit = true;
 }
 
@@ -248,34 +248,73 @@ static void load_helper_run(enum load load)
 		return;
 	}
 
+	lh.exit = false;
 	lh.load = load;
 
 	igt_fork_helper(&lh.igt_proc) {
+		const uint32_t bbe = MI_BATCH_BUFFER_END;
+		struct drm_i915_gem_exec_object2 object;
+		struct drm_i915_gem_execbuffer2 execbuf;
+		uint32_t fences[3];
 		uint32_t val = 0;
 
 		signal(SIGUSR1, load_helper_signal_handler);
 		signal(SIGUSR2, load_helper_signal_handler);
 
+		fences[0] = gem_create(drm_fd, 4096);
+		gem_write(drm_fd, fences[0], 0,  &bbe, sizeof(bbe));
+		fences[1] = gem_create(drm_fd, 4096);
+		gem_write(drm_fd, fences[1], 0,  &bbe, sizeof(bbe));
+		fences[2] = gem_create(drm_fd, 4096);
+		gem_write(drm_fd, fences[2], 0,  &bbe, sizeof(bbe));
+
+		memset(&execbuf, 0, sizeof(execbuf));
+		execbuf.buffers_ptr = (uintptr_t)&object;
+		execbuf.buffer_count = 1;
+		if (intel_gen(lh.devid) >= 6)
+			execbuf.flags = I915_EXEC_BLT;
+
+		igt_debug("Applying %s load...\n", lh.load ? "high" : "low");
+
 		while (!lh.exit) {
+			memset(&object, 0, sizeof(object));
+			object.handle = fences[val%3];
+
+			while (gem_bo_busy(drm_fd, object.handle))
+				usleep(100);
+
 			if (lh.load == HIGH)
 				intel_copy_bo(lh.batch, lh.dst, lh.src,
 					      LOAD_HELPER_BO_SIZE);
 
 			emit_store_dword_imm(val);
-			intel_batchbuffer_flush_on_ring(lh.batch, 0);
+			intel_batchbuffer_flush_on_ring(lh.batch,
+                                                        I915_EXEC_BLT);
 			val++;
 
+			gem_execbuf(drm_fd, &execbuf);
+
 			/* Lower the load by pausing after every submitted
 			 * write. */
 			if (lh.load == LOW)
 				usleep(LOAD_HELPER_PAUSE_USEC);
 		}
 
-		/* Map buffer to stall for write completion */
-		drm_intel_bo_map(lh.target_buffer, 0);
-		drm_intel_bo_unmap(lh.target_buffer);
+		/* Wait for completion without boosting */
+		usleep(1000);
+		while (gem_bo_busy(drm_fd, lh.target_buffer->handle))
+			usleep(1000);
 
 		igt_debug("load helper sent %u dword writes\n", val);
+		gem_close(drm_fd, fences[0]);
+		gem_close(drm_fd, fences[1]);
+		gem_close(drm_fd, fences[2]);
+
+		/* Idle/boost logic is tied with request retirement.
+		 * Speed up detection of idle state and ensure deboost
+		 * after removing load.
+		 */
+		igt_drop_caches_set(drm_fd, DROP_RETIRE);
 	}
 }
 
@@ -356,14 +395,18 @@ static int get_hw_rounded_freq(int target)
 		idx = MAX;
 
 	old_freq = freqs[idx];
-	writeval_nocheck(stuff[idx].filp, target);
+	writeval_nocheck(sysfs_files[idx].filp, target);
 	read_freqs(freqs);
 	ret = freqs[idx];
-	writeval_nocheck(stuff[idx].filp, old_freq);
+	writeval_nocheck(sysfs_files[idx].filp, old_freq);
 
 	return ret;
 }
 
+/*
+ * Modify softlimit MIN and MAX freqs to valid and invalid levels. Depending
+ * on subtest run different check after each modification.
+ */
 static void min_max_config(void (*check)(void), bool load_gpu)
 {
 	int fmid = (origfreqs[RPn] + origfreqs[RP0]) / 2;
@@ -380,78 +423,78 @@ static void min_max_config(void (*check)(void), bool load_gpu)
 	check();
 
 	igt_debug("\nSet min=RPn and max=RP0...\n");
-	writeval(stuff[MIN].filp, origfreqs[RPn]);
-	writeval(stuff[MAX].filp, origfreqs[RP0]);
+	writeval(sysfs_files[MIN].filp, origfreqs[RPn]);
+	writeval(sysfs_files[MAX].filp, origfreqs[RP0]);
 	if (load_gpu)
 		do_load_gpu();
 	check();
 
 	igt_debug("\nIncrease min to midpoint...\n");
-	writeval(stuff[MIN].filp, fmid);
+	writeval(sysfs_files[MIN].filp, fmid);
 	if (load_gpu)
 		do_load_gpu();
 	check();
 
 	igt_debug("\nIncrease min to RP0...\n");
-	writeval(stuff[MIN].filp, origfreqs[RP0]);
+	writeval(sysfs_files[MIN].filp, origfreqs[RP0]);
 	if (load_gpu)
 		do_load_gpu();
 	check();
 
 	igt_debug("\nIncrease min above RP0 (invalid)...\n");
-	writeval_inval(stuff[MIN].filp, origfreqs[RP0] + 1000);
+	writeval_inval(sysfs_files[MIN].filp, origfreqs[RP0] + 1000);
 	check();
 
 	igt_debug("\nDecrease max to RPn (invalid)...\n");
-	writeval_inval(stuff[MAX].filp, origfreqs[RPn]);
+	writeval_inval(sysfs_files[MAX].filp, origfreqs[RPn]);
 	check();
 
 	igt_debug("\nDecrease min to midpoint...\n");
-	writeval(stuff[MIN].filp, fmid);
+	writeval(sysfs_files[MIN].filp, fmid);
 	if (load_gpu)
 		do_load_gpu();
 	check();
 
 	igt_debug("\nDecrease min to RPn...\n");
-	writeval(stuff[MIN].filp, origfreqs[RPn]);
+	writeval(sysfs_files[MIN].filp, origfreqs[RPn]);
 	if (load_gpu)
 		do_load_gpu();
 	check();
 
 	igt_debug("\nDecrease min below RPn (invalid)...\n");
-	writeval_inval(stuff[MIN].filp, 0);
+	writeval_inval(sysfs_files[MIN].filp, 0);
 	check();
 
 	igt_debug("\nDecrease max to midpoint...\n");
-	writeval(stuff[MAX].filp, fmid);
+	writeval(sysfs_files[MAX].filp, fmid);
 	check();
 
 	igt_debug("\nDecrease max to RPn...\n");
-	writeval(stuff[MAX].filp, origfreqs[RPn]);
+	writeval(sysfs_files[MAX].filp, origfreqs[RPn]);
 	check();
 
 	igt_debug("\nDecrease max below RPn (invalid)...\n");
-	writeval_inval(stuff[MAX].filp, 0);
+	writeval_inval(sysfs_files[MAX].filp, 0);
 	check();
 
 	igt_debug("\nIncrease min to RP0 (invalid)...\n");
-	writeval_inval(stuff[MIN].filp, origfreqs[RP0]);
+	writeval_inval(sysfs_files[MIN].filp, origfreqs[RP0]);
 	check();
 
 	igt_debug("\nIncrease max to midpoint...\n");
-	writeval(stuff[MAX].filp, fmid);
+	writeval(sysfs_files[MAX].filp, fmid);
 	check();
 
 	igt_debug("\nIncrease max to RP0...\n");
-	writeval(stuff[MAX].filp, origfreqs[RP0]);
+	writeval(sysfs_files[MAX].filp, origfreqs[RP0]);
 	check();
 
 	igt_debug("\nIncrease max above RP0 (invalid)...\n");
-	writeval_inval(stuff[MAX].filp, origfreqs[RP0] + 1000);
+	writeval_inval(sysfs_files[MAX].filp, origfreqs[RP0] + 1000);
 	check();
 
-	writeval(stuff[MIN].filp, origfreqs[MIN]);
-	writeval(stuff[MAX].filp, origfreqs[MAX]);
+	writeval(sysfs_files[MIN].filp, origfreqs[MIN]);
+	writeval(sysfs_files[MAX].filp, origfreqs[MAX]);
 }
 
 static void basic_check(void)
@@ -460,11 +503,11 @@ static void basic_check(void)
 
 	read_freqs(freqs);
 	dump(freqs);
-	checkit(freqs);
+	check_freq_constraints(freqs);
 }
 
-#define IDLE_WAIT_TIMESTEP_MSEC 100
-#define IDLE_WAIT_TIMEOUT_MSEC 10000
+#define IDLE_WAIT_TIMESTEP_MSEC 250
+#define IDLE_WAIT_TIMEOUT_MSEC 2500
 static void idle_check(void)
 {
 	int freqs[NUMFREQ];
@@ -475,7 +518,7 @@ static void idle_check(void)
 	do {
 		read_freqs(freqs);
 		dump(freqs);
-		checkit(freqs);
+		check_freq_constraints(freqs);
 		if (freqs[CUR] == freqs[RPn])
 			break;
 		usleep(1000 * IDLE_WAIT_TIMESTEP_MSEC);
@@ -498,158 +541,119 @@ static void loaded_check(void)
 	do {
 		read_freqs(freqs);
 		dump(freqs);
-		checkit(freqs);
-		if (freqs[CUR] == freqs[MAX])
+		check_freq_constraints(freqs);
+		if (freqs[CUR] >= freqs[MAX])
 			break;
 		usleep(1000 * LOADED_WAIT_TIMESTEP_MSEC);
 		wait += LOADED_WAIT_TIMESTEP_MSEC;
 	} while (wait < LOADED_WAIT_TIMEOUT_MSEC);
 
-	igt_assert_eq(freqs[CUR], freqs[MAX]);
+	igt_assert_lte(freqs[MAX], freqs[CUR]);
 	igt_debug("Required %d msec to reach cur=max\n", wait);
 }
 
-#define STABILIZE_WAIT_TIMESTEP_MSEC 100
-#define STABILIZE_WAIT_TIMEOUT_MSEC 10000
-static void stabilize_check(int *freqs)
+#define STABILIZE_WAIT_TIMESTEP_MSEC 250
+#define STABILIZE_WAIT_TIMEOUT_MSEC 15000
+static void stabilize_check(int *out)
 {
+	int freqs[NUMFREQ];
 	int wait = 0;
 
+	/* Monitor frequencies until HW will stabilize cur frequency.
+	 * It should happen within allotted time */
+	read_freqs(freqs);
+	dump(freqs);
+	usleep(1000 * STABILIZE_WAIT_TIMESTEP_MSEC);
 	do {
-		read_freqs(freqs);
-		dump(freqs);
-		usleep(1000 * STABILIZE_WAIT_TIMESTEP_MSEC);
+		read_freqs(out);
+		dump(out);
+
+		if (memcmp(freqs, out, sizeof(freqs)) == 0)
+			break;
+
+		memcpy(freqs, out, sizeof(freqs));
 		wait += STABILIZE_WAIT_TIMESTEP_MSEC;
 	} while (wait < STABILIZE_WAIT_TIMEOUT_MSEC);
 
 	igt_debug("Waited %d msec to stabilize cur\n", wait);
 }
 
-/*
- * reset - test that turbo works across a ring stop
- *
- * METHOD
- *   Apply a low GPU load, collect the resulting frequencies, then stop
- *   the GPU by stopping the rings.  Apply alternating high and low loads
- *   following the restart, comparing against the previous low load freqs
- *   and whether the GPU ramped to max freq successfully.  Finally check
- *   that we return to idle at the end.
- *
- * EXPECTED RESULTS
- *   Low load freqs match, high load freqs reach max, and GPU returns to
- *   idle at the end.
- *
- * FAILURES
- *    Failures here could indicate turbo doesn't work across a ring stop
- *    or that load generation routines don't successfully generate stable
- *    or maximal GPU loads.  It could also indicate a thermal limit if the
- *    GPU isn't able to reach its maximum frequency.
- */
-static void reset(void)
+static void boost_freq(int fd, int *boost_freqs)
 {
-	int pre_freqs[NUMFREQ];
-	int post_freqs[NUMFREQ];
+	int64_t timeout = 1;
+	igt_spin_t *load;
+	unsigned int engine;
+	int fmid = (origfreqs[RPn] + origfreqs[RP0]) / 2;
 
-	/*
-	 * quiescent_gpu upsets the gpu and makes it get pegged to max somehow.
-	 * Don't ask.
-	 */
-	sleep(10);
+	fmid = get_hw_rounded_freq(fmid);
+	/* Set max freq to less then boost freq */
+	writeval(sysfs_files[MAX].filp, fmid);
 
-	igt_debug("Apply low load...\n");
-	load_helper_run(LOW);
-	stabilize_check(pre_freqs);
+	/* Put boost on the same engine as low load */
+	engine = I915_EXEC_RENDER;
+	if (intel_gen(lh.devid) >= 6)
+		engine = I915_EXEC_BLT;
+	load = igt_spin_batch_new(fd, 0, engine, 0);
+	/* Waiting will grant us a boost to maximum */
+	gem_wait(fd, load->handle, &timeout);
 
-	igt_debug("Stop rings...\n");
-	igt_set_stop_rings(STOP_RING_DEFAULTS);
-	while (igt_get_stop_rings())
-		usleep(1000 * 100);
-	igt_debug("Ring stop cleared\n");
+	read_freqs(boost_freqs);
+	dump(boost_freqs);
 
-	igt_debug("Apply high load...\n");
-	load_helper_set_load(HIGH);
-	loaded_check();
+	/* Avoid downlocking till boost request is pending */
+	igt_spin_batch_end(load);
+	gem_sync(fd, load->handle);
+	igt_spin_batch_free(fd, load);
 
-	igt_debug("Apply low load...\n");
-	load_helper_set_load(LOW);
-	stabilize_check(post_freqs);
-	matchit(pre_freqs, post_freqs);
-
-	igt_debug("Apply high load...\n");
-	load_helper_set_load(HIGH);
-	loaded_check();
-
-	igt_debug("Removing load...\n");
-	load_helper_stop();
-	idle_check();
+	/* Set max freq to original softmax */
+	writeval(sysfs_files[MAX].filp, origfreqs[MAX]);
 }
 
-/*
- * blocking - test that GPU returns to idle after a forced blocking boost
- *   and a low GPU load.  Frequencies resulting from the low load are also
- *   expected to match.o
- *
- * METHOD
- *   Collect frequencies resulting from a low GPU load and compare with
- *   frequencies collected after a quiesce and a second low load, then
- *   verify idle.
- *
- * EXPECTED RESULTS
- *   Frequencies match and GPU successfully returns to idle afterward.
- *
- * FAILURES
- *   Failures in this test could be due to several possible bugs:
- *     - load generation creates unstable frequencies, though stabilize_check()
- *       is supposed to catch this
- *     - quiescent_gpu() call does not boost GPU to max freq
- *     - frequency ramp down is too slow, causing second set of collected
- *       frequencies to be higher than the first
- */
-static void blocking(void)
+static void waitboost(int fd, bool reset)
 {
 	int pre_freqs[NUMFREQ];
+	int boost_freqs[NUMFREQ];
 	int post_freqs[NUMFREQ];
 
-	int fd = drm_open_driver(DRIVER_INTEL);
-	igt_assert_lte(0, fd);
-
-	/*
-	 * quiescent_gpu upsets the gpu and makes it get pegged to max somehow.
-	 * Don't ask.
-	 */
-	sleep(10);
+	load_helper_run(LOW);
 
 	igt_debug("Apply low load...\n");
-	load_helper_run(LOW);
+	sleep(1);
 	stabilize_check(pre_freqs);
-	load_helper_stop();
 
-	sleep(5);
+	if (reset) {
+		igt_debug("Reset gpu...\n");
+		igt_force_gpu_reset(fd);
+		sleep(1);
+	}
 
-	igt_debug("Kick gpu hard ...\n");
-	/* This relies on the blocking waits in quiescent_gpu and the kernel
-	 * boost logic to ramp the gpu to full load. */
-	gem_quiescent_gpu(fd);
-	gem_quiescent_gpu(fd);
+	/* When we wait upon the GPU, we want to temporarily boost it
+	 * to maximum.
+	 */
+	boost_freq(fd, boost_freqs);
 
 	igt_debug("Apply low load again...\n");
-	load_helper_run(LOW);
+	sleep(1);
 	stabilize_check(post_freqs);
-	load_helper_stop();
-	matchit(pre_freqs, post_freqs);
 
 	igt_debug("Removing load...\n");
+	load_helper_stop();
 	idle_check();
+
+	igt_assert_lt(pre_freqs[CUR], pre_freqs[MAX]);
+	igt_assert_eq(boost_freqs[CUR], boost_freqs[BOOST]);
+	igt_assert_lt(post_freqs[CUR], post_freqs[MAX]);
+
 }
 
 static void pm_rps_exit_handler(int sig)
 {
-	if (origfreqs[MIN] > readval(stuff[MAX].filp)) {
-		writeval(stuff[MAX].filp, origfreqs[MAX]);
-		writeval(stuff[MIN].filp, origfreqs[MIN]);
+	if (origfreqs[MIN] > readval(sysfs_files[MAX].filp)) {
+		writeval(sysfs_files[MAX].filp, origfreqs[MAX]);
+		writeval(sysfs_files[MIN].filp, origfreqs[MIN]);
 	} else {
-		writeval(stuff[MIN].filp, origfreqs[MIN]);
-		writeval(stuff[MAX].filp, origfreqs[MAX]);
+		writeval(sysfs_files[MIN].filp, origfreqs[MIN]);
+		writeval(sysfs_files[MAX].filp, origfreqs[MAX]);
 	}
 
 	load_helper_deinit();
@@ -662,25 +666,28 @@ igt_main
 
 	igt_fixture {
 		const int device = drm_get_card();
-		struct junk *junk = stuff;
+		struct sysfs_file *sysfs_file = sysfs_files;
 		int ret;
 
 		/* Use drm_open_driver to verify device existence */
 		drm_fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(drm_fd);
+		igt_require(gem_can_store_dword(drm_fd, 0));
 
 		do {
 			int val = -1;
 			char *path;
-			ret = asprintf(&path, sysfs_base_path, device, junk->name);
-			igt_assert(ret != -1);
-			junk->filp = fopen(path, junk->mode);
-			igt_require(junk->filp);
-			setbuf(junk->filp, NULL);
 
-			val = readval(junk->filp);
+			ret = asprintf(&path, sysfs_base_path, device, sysfs_file->name);
+			igt_assert(ret != -1);
+			sysfs_file->filp = fopen(path, sysfs_file->mode);
+			igt_require(sysfs_file->filp);
+			setbuf(sysfs_file->filp, NULL);
+
+			val = readval(sysfs_file->filp);
 			igt_assert(val >= 0);
-			junk++;
-		} while(junk->name != NULL);
+			sysfs_file++;
+		} while (sysfs_file->name != NULL);
 
 		read_freqs(origfreqs);
 
@@ -692,18 +699,25 @@ igt_main
 	igt_subtest("basic-api")
 		min_max_config(basic_check, false);
 
+	/* Verify the constraints, check if we can reach idle */
 	igt_subtest("min-max-config-idle")
 		min_max_config(idle_check, true);
 
+	/* Verify the constraints with high load, check if we can reach max */
 	igt_subtest("min-max-config-loaded") {
 		load_helper_run(HIGH);
 		min_max_config(loaded_check, false);
 		load_helper_stop();
 	}
 
-	igt_subtest("reset")
-		reset();
+	/* Checks if we achieve boost using gem_wait */
+	igt_subtest("waitboost")
+		waitboost(drm_fd, false);
 
-	igt_subtest("blocking")
-		blocking();
+	/* Test boost frequency after GPU reset */
+	igt_subtest("reset") {
+		igt_hang_t hang = igt_allow_hang(drm_fd, 0, 0);
+		waitboost(drm_fd, true);
+		igt_disallow_hang(drm_fd, hang);
+	}
 }
