@@ -36,9 +36,7 @@
 #include "intel_io.h"
 #include "igt_debugfs.h"
 #include "drmtest.h"
-
-#define min(a,b) ((a) < (b) ? (a) : (b))
-#define max(a,b) ((a) > (b) ? (a) : (b))
+#include "igt_aux.h"
 
 enum test {
 	TEST_INVALID,
@@ -59,6 +57,9 @@ enum test {
 };
 
 static uint32_t vlv_offset;
+static uint16_t pipe_offset[3] = { 0, 0x1000, 0x2000, };
+
+#define PIPE_REG(pipe, reg_a) (pipe_offset[(pipe)] + (reg_a))
 
 static volatile bool quit;
 
@@ -69,22 +70,27 @@ static void sighandler(int x)
 
 static uint16_t read_reg_16(uint32_t reg)
 {
-	return *(volatile uint16_t *)((volatile char*)mmio + vlv_offset + reg);
+	return INREG16(vlv_offset + reg);
 }
 
 static uint32_t read_reg(uint32_t reg)
 {
-	return *(volatile uint32_t *)((volatile char*)mmio + vlv_offset + reg);
+	return INREG(vlv_offset + reg);
 }
 
 static void write_reg_16(uint32_t reg, uint16_t val)
 {
-	*(volatile uint16_t *)((volatile char*)mmio + vlv_offset + reg) = val;
+	OUTREG16(vlv_offset + reg, val);
 }
 
 static void write_reg(uint32_t reg, uint32_t val)
 {
-	*(volatile uint32_t *)((volatile char*)mmio + vlv_offset + reg) = val;
+	OUTREG(vlv_offset + reg, val);
+}
+
+static char pipe_name(int pipe)
+{
+	return pipe + 'A';
 }
 
 static int pipe_to_plane(uint32_t devid, int pipe)
@@ -94,13 +100,21 @@ static int pipe_to_plane(uint32_t devid, int pipe)
 
 	switch (pipe) {
 	case 0:
-		if ((read_reg(DSPACNTR) & DISPPLANE_SEL_PIPE_MASK) == DISPPLANE_SEL_PIPE_B)
-			return 1;
-		return 0;
-	case 1:
 		if ((read_reg(DSPACNTR) & DISPPLANE_SEL_PIPE_MASK) == DISPPLANE_SEL_PIPE_A)
 			return 0;
-		return 1;
+		if ((read_reg(DSPBCNTR) & DISPPLANE_SEL_PIPE_MASK) == DISPPLANE_SEL_PIPE_A)
+			return 1;
+		fprintf(stderr, "no plane assigned to pipe %c, assuming %c\n",
+			pipe_name(pipe), pipe_name(pipe));
+		return pipe;
+	case 1:
+		if ((read_reg(DSPACNTR) & DISPPLANE_SEL_PIPE_MASK) == DISPPLANE_SEL_PIPE_B)
+			return 0;
+		if ((read_reg(DSPBCNTR) & DISPPLANE_SEL_PIPE_MASK) == DISPPLANE_SEL_PIPE_B)
+			return 1;
+		fprintf(stderr, "no plane assigned to pipe %c, assuming %c\n",
+			pipe_name(pipe), pipe_name(pipe));
+		return pipe;
 	}
 
 	assert(0);
@@ -111,95 +125,29 @@ static int pipe_to_plane(uint32_t devid, int pipe)
 static uint32_t dspoffset_reg(uint32_t devid, int pipe)
 {
 	bool use_tileoff;
-
-	pipe = pipe_to_plane(devid, pipe);
+	int plane = pipe_to_plane(devid, pipe);
 
 	if (IS_GEN2(devid) || IS_GEN3(devid))
 		use_tileoff = false;
-	if (IS_HASWELL(devid) || IS_BROADWELL(devid))
+	else if (IS_HASWELL(devid) || IS_BROADWELL(devid))
 		use_tileoff = true;
-	else {
-		switch (pipe) {
-		case 0:
-			use_tileoff = read_reg(DSPACNTR) & DISPLAY_PLANE_TILED;
-			break;
-		case 1:
-			use_tileoff = read_reg(DSPBCNTR) & DISPLAY_PLANE_TILED;
-			break;
-		case 2:
-			use_tileoff = read_reg(DSPCCNTR) & DISPLAY_PLANE_TILED;
-			break;
-		}
-	}
+	else
+		use_tileoff = read_reg(PIPE_REG(plane, DSPACNTR)) & DISPLAY_PLANE_TILED;
 
-	if (use_tileoff) {
-		switch (pipe) {
-		case 0:
-			return DSPATILEOFF;
-		case 1:
-			return DSPBTILEOFF;
-		case 2:
-			return DSPCTILEOFF;
-		}
-	} else {
-		switch (pipe) {
-		case 0:
-			return DSPABASE;
-		case 1:
-			return DSPBBASE;
-		case 2:
-			return DSPCBASE;
-		}
-	}
-
-	assert(0);
-
-	return 0;
+	if (use_tileoff)
+		return PIPE_REG(plane, DSPATILEOFF);
+	else
+		return PIPE_REG(plane, DSPABASE);
 }
 
 static uint32_t dspsurf_reg(uint32_t devid, int pipe)
 {
-	pipe = pipe_to_plane(devid, pipe);
+	int plane = pipe_to_plane(devid, pipe);
 
-	if (IS_GEN2(devid) || IS_GEN3(devid)) {
-		switch (pipe) {
-		case 0:
-			return DSPABASE;
-		case 1:
-			return DSPBBASE;
-		case 2:
-			return DSPCBASE;
-		}
-	} else {
-		switch (pipe) {
-		case 0:
-			return DSPASURF;
-		case 1:
-			return DSPBSURF;
-		case 2:
-			return DSPCSURF;
-		}
-	}
-
-	assert(0);
-
-	return 0;
-}
-
-static uint32_t dsl_reg(int pipe)
-{
-	switch (pipe) {
-	case 0:
-		return PIPEA_DSL;
-	case 1:
-		return PIPEB_DSL;
-	case 2:
-		return PIPEC_DSL;
-	}
-
-	assert(0);
-
-	return 0;
+	if (IS_GEN2(devid) || IS_GEN3(devid))
+		return PIPE_REG(plane, DSPABASE);
+	else
+		return PIPE_REG(plane, DSPASURF);
 }
 
 static void poll_pixel_pipestat(int pipe, int bit, uint32_t *min, uint32_t *max, const int count)
@@ -207,20 +155,9 @@ static void poll_pixel_pipestat(int pipe, int bit, uint32_t *min, uint32_t *max,
 	uint32_t pix, pix1, pix2, iir, iir1, iir2, iir_bit, iir_mask;
 	int i = 0;
 
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-		iir_bit = 1 << bit;
-		iir = PIPEASTAT;
-		break;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-		iir_bit = 1 << bit;
-		iir = PIPEBSTAT;
-		break;
-	default:
-		return;
-	}
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
+	iir_bit = 1 << bit;
+	iir = PIPE_REG(pipe, PIPEASTAT);
 
 	iir_mask = read_reg(iir) & 0x7fff0000;
 
@@ -256,17 +193,7 @@ static void poll_pixel_iir_gen3(int pipe, int bit, uint32_t *min, uint32_t *max,
 	int i = 0;
 
 	bit = 1 << bit;
-
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-		break;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-		break;
-	default:
-		return;
-	}
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
 
 	imr_save = read_reg(IMR);
 	ier_save = read_reg(IER);
@@ -308,16 +235,7 @@ static void poll_pixel_framecount_gen3(int pipe, uint32_t *min, uint32_t *max, c
 	uint32_t pix, pix1, pix2, frm1, frm2;
 	int i = 0;
 
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-		break;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-		break;
-	default:
-		return;
-	}
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
 
 	while (!quit) {
 		pix1 = read_reg(pix);
@@ -346,17 +264,7 @@ static void poll_pixel_pan(uint32_t devid, int pipe, int target_pixel, int targe
 	uint32_t saved, surf = 0;
 	int i = 0;
 
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-		break;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-		break;
-	default:
-		return;
-	}
-
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
 	surf = dspoffset_reg(devid, pipe);
 
 	saved = read_reg(surf);
@@ -390,19 +298,11 @@ static void poll_pixel_pan(uint32_t devid, int pipe, int target_pixel, int targe
 static void poll_pixel_flip(uint32_t devid, int pipe, int target_pixel, int target_fuzz,
 			    uint32_t *min, uint32_t *max, const int count)
 {
-	uint32_t pix, pix1, pix2;
+	uint32_t pix, pix1 = 0, pix2 = 0;
 	uint32_t saved, surf = 0;
 	int i = 0;
 
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-	default:
-		return;
-	}
-
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
 	surf = dspsurf_reg(devid, pipe);
 
 	saved = read_reg(surf);
@@ -414,7 +314,7 @@ static void poll_pixel_flip(uint32_t devid, int pipe, int target_pixel, int targ
 				break;
 		}
 
-		write_reg(surf, saved+4096);
+		write_reg(surf, saved+128*1024);
 
 		while (!quit){
 			pix2 = read_reg(pix) & PIPE_PIXEL_MASK;
@@ -438,16 +338,7 @@ static void poll_pixel_wrap(int pipe, uint32_t *min, uint32_t *max, const int co
 	uint32_t pix, pix1, pix2;
 	int i = 0;
 
-	switch (pipe) {
-	case 0:
-		pix = PIPEAFRAMEPIXEL;
-		break;
-	case 1:
-		pix = PIPEBFRAMEPIXEL;
-		break;
-	default:
-		return;
-	}
+	pix = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
 
 	while (!quit) {
 		pix1 = read_reg(pix);
@@ -473,20 +364,9 @@ static void poll_dsl_pipestat(int pipe, int bit,
 	bool field1, field2;
 	int i[2] = {};
 
-	switch (pipe) {
-	case 0:
-		iir_bit = 1 << bit;
-		iir = PIPEASTAT;
-		break;
-	case 1:
-		iir_bit = 1 << bit;
-		iir = PIPEBSTAT;
-		break;
-	default:
-		return;
-	}
-
-	dsl = dsl_reg(pipe);
+	iir_bit = 1 << bit;
+	iir = PIPE_REG(pipe, PIPEASTAT);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	iir_mask = read_reg(iir) & 0x7fff0000;
 
@@ -530,8 +410,7 @@ static void poll_dsl_iir_gen2(int pipe, int bit,
 	int i[2] = {};
 
 	bit = 1 << bit;
-
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	imr_save = read_reg_16(IMR);
 	ier_save = read_reg_16(IER);
@@ -582,8 +461,7 @@ static void poll_dsl_iir_gen3(int pipe, int bit,
 	int i[2] = {};
 
 	bit = 1 << bit;
-
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	imr_save = read_reg(IMR);
 	ier_save = read_reg(IER);
@@ -635,10 +513,9 @@ static void poll_dsl_deiir(uint32_t devid, int pipe, int bit,
 	int i[2] = {};
 
 	bit = 1 << bit;
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
-	dsl = dsl_reg(pipe);
-
-	if (IS_GEN8(devid)) {
+	if (intel_gen(devid) >= 8) {
 		iir = GEN8_DE_PIPE_IIR(pipe);
 		ier = GEN8_DE_PIPE_IER(pipe);
 		imr = GEN8_DE_PIPE_IMR(pipe);
@@ -695,21 +572,8 @@ static void poll_dsl_framecount_g4x(int pipe, uint32_t *min, uint32_t *max, cons
 	bool field1, field2;
 	int i[2] = {};
 
-	switch (pipe) {
-	case 0:
-		frm = PIPEAFRMCOUNT_G4X;
-		break;
-	case 1:
-		frm = PIPEBFRMCOUNT_G4X;
-		break;
-	case 2:
-		frm = PIPECFRMCOUNT_G4X;
-		break;
-	default:
-		return;
-	}
-
-	dsl = dsl_reg(pipe);
+	frm = PIPE_REG(pipe, PIPEAFRMCOUNT_G4X);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	while (!quit) {
 		dsl1 = read_reg(dsl);
@@ -743,21 +607,8 @@ static void poll_dsl_flipcount_g4x(uint32_t devid, int pipe,
 	bool field1, field2;
 	int i[2] = {};
 
-	switch (pipe) {
-	case 0:
-		flp = PIPEAFLIPCOUNT_G4X;
-		break;
-	case 1:
-		flp = PIPEBFLIPCOUNT_G4X;
-		break;
-	case 2:
-		flp = PIPECFLIPCOUNT_G4X;
-		break;
-	default:
-		return;
-	}
-
-	dsl = dsl_reg(pipe);
+	flp = PIPE_REG(pipe, PIPEAFLIPCOUNT_G4X);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 	surf = dspsurf_reg(devid, pipe);
 
 	while (!quit) {
@@ -815,18 +666,8 @@ static void poll_dsl_framecount_gen3(int pipe, uint32_t *min, uint32_t *max, con
 	bool field1, field2;
 	int i[2] = {};
 
-	switch (pipe) {
-	case 0:
-		frm = PIPEAFRAMEPIXEL;
-		break;
-	case 1:
-		frm = PIPEBFRAMEPIXEL;
-		break;
-	default:
-		return;
-	}
-
-	dsl = dsl_reg(pipe);
+	frm = PIPE_REG(pipe, PIPEAFRAMEPIXEL);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	while (!quit) {
 		dsl1 = read_reg(dsl);
@@ -861,7 +702,7 @@ static void poll_dsl_pan(uint32_t devid, int pipe, int target_scanline, int targ
 	uint32_t saved, surf = 0;
 	int i[2] = {};
 
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 	surf = dspoffset_reg(devid, pipe);
 
 	saved = read_reg(surf);
@@ -908,7 +749,7 @@ static void poll_dsl_flip(uint32_t devid, int pipe, int target_scanline, int tar
 	uint32_t saved, surf = 0;
 	int i[2] = {};
 
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 	surf = dspsurf_reg(devid, pipe);
 
 	saved = read_reg(surf);
@@ -922,7 +763,7 @@ static void poll_dsl_flip(uint32_t devid, int pipe, int target_scanline, int tar
 				break;
 		}
 
-		write_reg(surf, saved+4096);
+		write_reg(surf, saved+128*1024);
 
 		while (!quit) {
 			dsl2 = read_reg(dsl);
@@ -954,27 +795,14 @@ static void poll_dsl_surflive(uint32_t devid, int pipe,
 	bool field1 = false, field2 = false;
 	int i[2] = {};
 
-	switch (pipe) {
-	case 0:
-		surflive = DSPASURFLIVE;
-		break;
-	case 1:
-		surflive = DSPBSURFLIVE;
-		break;
-	case 2:
-		surflive = DSPCSURFLIVE;
-		break;
-	default:
-		return;
-	}
-
-	dsl = dsl_reg(pipe);
+	surflive = PIPE_REG(pipe, DSPASURFLIVE);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 	surf = dspsurf_reg(devid, pipe);
 
 	saved = read_reg(surf);
 
 	surf1 = saved & ~0xfff;
-	surf2 = surf1 + 4096;
+	surf2 = surf1 + 128*1024;
 
 	while (!quit) {
 		write_reg(surf, surf2);
@@ -1019,7 +847,7 @@ static void poll_dsl_wrap(int pipe, uint32_t *min, uint32_t *max, const int coun
 	bool field1, field2;
 	int i[2] = {};
 
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	while (!quit) {
 		dsl1 = read_reg(dsl);
@@ -1050,7 +878,7 @@ static void poll_dsl_field(int pipe, uint32_t *min, uint32_t *max, const int cou
 	bool field1, field2;
 	int i[2] = {};
 
-	dsl = dsl_reg(pipe);
+	dsl = PIPE_REG(pipe, PIPEA_DSL);
 
 	while (!quit) {
 		dsl1 = read_reg(dsl);
@@ -1071,14 +899,9 @@ static void poll_dsl_field(int pipe, uint32_t *min, uint32_t *max, const int cou
 	}
 }
 
-static char pipe_name(int pipe)
-{
-	return pipe + 'A';
-}
-
 static const char *test_name(enum test test, int pipe, int bit, bool test_pixel_count)
 {
-	static char str[32];
+	static char str[64];
 	const char *type = test_pixel_count ? "pixel" : "dsl";
 
 	switch (test) {
@@ -1123,7 +946,7 @@ static const char *test_name(enum test test, int pipe, int bit, bool test_pixel_
 	}
 }
 
-static void usage(const char *name)
+static void __attribute__((noreturn)) usage(const char *name)
 {
 	fprintf(stderr, "Usage: %s [options]\n"
 		" -t,--test <pipestat|iir|framecount|flipcount|pan|flip|surflive|wrap|field>\n"
@@ -1138,10 +961,11 @@ static void usage(const char *name)
 
 int main(int argc, char *argv[])
 {
-	int fd, i;
+	int i;
 	int pipe = 0, bit = 0, target_scanline = 0, target_fuzz = 1;
 	bool test_pixelcount = false;
 	uint32_t devid;
+	int drm_fd;
 	uint32_t min[2*128] = {};
 	uint32_t max[2*128] = {};
 	uint32_t a, b;
@@ -1187,7 +1011,17 @@ int main(int argc, char *argv[])
 				usage(argv[0]);
 			break;
 		case 'p':
-			pipe = atoi(optarg);
+			if (optarg[1] != '\0')
+				usage(argv[0]);
+			pipe = optarg[0];
+			if (pipe >= 'a')
+				pipe -= 'a';
+			else if (pipe >= 'A')
+				pipe -= 'A';
+			else if (pipe >= '0')
+				pipe -= '0';
+			else
+				usage(argv[0]);
 			if (pipe < 0 || pipe > 2)
 				usage(argv[0]);
 			break;
@@ -1212,9 +1046,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	fd = drm_open_any();
-	devid = intel_get_drm_devid(fd);
-	close(fd);
+	devid = intel_get_pci_device()->device_id;
 
 	/*
 	 * check if the requires registers are
@@ -1264,11 +1096,14 @@ int main(int argc, char *argv[])
 		default:
 			usage(argv[0]);
 		}
-	} else if (IS_G4X(devid) || IS_VALLEYVIEW(devid)) {
-		if (IS_VALLEYVIEW(devid))
+	} else if (IS_G4X(devid) ||
+		   IS_VALLEYVIEW(devid) || IS_CHERRYVIEW(devid)) {
+		if (IS_VALLEYVIEW(devid) || IS_CHERRYVIEW(devid))
 			vlv_offset = 0x180000;
+		if (IS_CHERRYVIEW(devid))
+			pipe_offset[2] = 0x4000;
 
-		if (pipe > 1)
+		if (pipe > 1 && !IS_CHERRYVIEW(devid))
 			usage(argv[0]);
 
 		if (test_pixelcount)
@@ -1318,7 +1153,7 @@ int main(int argc, char *argv[])
 		default:
 			usage(argv[0]);
 		}
-	} else if (IS_GEN8(devid)) {
+	} else if (intel_gen(devid) >= 8) {
 		if (test_pixelcount)
 			usage(argv[0]);
 
@@ -1352,7 +1187,10 @@ int main(int argc, char *argv[])
 		break;
 	}
 
-	intel_register_access_init(intel_get_pci_device(), 0);
+	/* Just to make sure we open the right debugfs files */
+	drm_fd = drm_open_driver_master(DRIVER_INTEL);
+
+	intel_register_access_init(intel_get_pci_device(), 0, drm_fd);
 
 	printf("%s?\n", test_name(test, pipe, bit, test_pixelcount));
 
@@ -1429,6 +1267,8 @@ int main(int argc, char *argv[])
 
 	intel_register_access_fini();
 
+	close(drm_fd);
+
 	if (quit)
 		return 0;
 
