@@ -38,7 +38,6 @@
  * decode, but never crash in the process.
  */
 
-#define _GNU_SOURCE
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -440,8 +439,8 @@ static void decode(struct drm_intel_decode *ctx,
 		   const char *ring_name,
 		   uint64_t gtt_offset,
 		   uint32_t head_offset,
-		   uint32_t *data,
-		   int *count)
+		   uint32_t *data, int *count,
+		   int decode)
 {
 	if (!*count)
 		return;
@@ -455,8 +454,15 @@ static void decode(struct drm_intel_decode *ctx,
 		       (unsigned)((head_offset + gtt_offset) & 0xffffffff));
 	printf("\n");
 
-	drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset, *count);
-	drm_intel_decode(ctx);
+	if (decode) {
+		drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset,
+						   *count);
+		drm_intel_decode(ctx);
+	} else {
+		for (int i = 0; i + 4 <= *count; i += 4)
+			printf("[%04x] %08x %08x %08x %08x\n",
+			       4*i, data[i], data[i+1], data[i+2], data[i+3]);
+	}
 	*count = 0;
 }
 
@@ -475,7 +481,7 @@ static int zlib_inflate(uint32_t **ptr, int len)
 
 	out = malloc(128*4096); /* approximate obj size */
 	zstream.next_out = out;
-	zstream.avail_out = 40*len;
+	zstream.avail_out = 128*4096;
 
 	do {
 		switch (inflate(&zstream, Z_SYNC_FLUSH)) {
@@ -554,103 +560,94 @@ read_data_file(FILE *file)
 	int head_idx = 0;
 	int num_rings = 0;
 	long long unsigned fence;
-	int data_size = 0, count = 0, line_number = 0, matched;
+	int data_size = 0, count = 0, matched;
 	char *line = NULL;
 	size_t line_size;
 	uint32_t offset, value, ring_length = 0;
-	uint64_t gtt_offset = 0, new_gtt_offset;
+	uint64_t gtt_offset = 0;
 	uint32_t head_offset = -1;
 	const char *buffer_name = "batch buffer";
 	char *ring_name = NULL;
+	int do_decode = 1;
 
 	while (getline(&line, &line_size, file) > 0) {
 		char *dashes;
-		line_number++;
-
-		dashes = strstr(line, "---");
-		if (dashes) {
-			uint32_t lo, hi;
-			char *new_ring_name = malloc(dashes - line);
-			strncpy(new_ring_name, line, dashes - line);
-			new_ring_name[dashes - line - 1] = '\0';
-
-			matched = sscanf(dashes, "--- gtt_offset = 0x%08x %08x\n",
-					 &hi, &lo);
-			if (matched > 0) {
-				new_gtt_offset = hi;
-				if (matched == 2) {
-					new_gtt_offset <<= 32;
-					new_gtt_offset |= lo;
-				}
-
-				decode(decode_ctx,
-				       buffer_name, ring_name,
-				       gtt_offset, head_offset,
-				       data, &count);
-				gtt_offset = new_gtt_offset;
-				head_offset = -1;
-				free(ring_name);
-				ring_name = new_ring_name;
-				buffer_name = "batch buffer";
-				continue;
-			}
-
-			matched = sscanf(dashes, "--- ringbuffer = 0x%08x %08x\n",
-					 &hi, &lo);
-			if (matched > 0) {
-				new_gtt_offset = hi;
-				if (matched == 2) {
-					new_gtt_offset <<= 32;
-					new_gtt_offset |= lo;
-				}
-
-				decode(decode_ctx,
-				       buffer_name, ring_name,
-				       gtt_offset, head_offset,
-				       data, &count);
-				gtt_offset = new_gtt_offset;
-				if (head_idx < num_rings)
-					head_offset = head[head_idx++];
-				else
-					head_offset = -1;
-				free(ring_name);
-				ring_name = new_ring_name;
-				buffer_name = "ring buffer";
-				continue;
-			}
-
-			matched = sscanf(dashes, "--- HW Context = 0x%08x %08x\n",
-					 &hi, &lo);
-			if (matched > 0) {
-				new_gtt_offset = hi;
-				if (matched == 2) {
-					new_gtt_offset <<= 32;
-					new_gtt_offset |= lo;
-				}
-
-				decode(decode_ctx,
-				       buffer_name, ring_name,
-				       gtt_offset, head_offset,
-				       data, &count);
-				gtt_offset = new_gtt_offset;
-				head_offset = -1;
-				free(ring_name);
-				ring_name = new_ring_name;
-				buffer_name = "HW Context";
-				continue;
-			}
-		}
 
 		if (line[0] == ':' || line[0] == '~') {
 			count = ascii85_decode(line+1, &data, line[0] == ':');
-			if (count == 0) {
-				fprintf(stderr, "ASCII85 decode failed.\n");
-				exit(1);
-			}
+			if (count == 0)
+				fprintf(stderr, "ASCII85 decode failed (%s - %s).\n",
+					ring_name, buffer_name);
 			decode(decode_ctx,
 			       buffer_name, ring_name,
 			       gtt_offset, head_offset,
-			       data, &count);
+			       data, &count, do_decode);
+			continue;
+		}
+
+		dashes = strstr(line, "---");
+		if (dashes) {
+			const struct {
+				const char *match;
+				const char *name;
+				int do_decode;
+			} buffers[] = {
+				{ "ringbuffer", "ring", 1 },
+				{ "gtt_offset", "batch", 1 },
+				{ "hw context", "HW context", 1 },
+				{ "hw status", "HW status", 0 },
+				{ "wa context", "WA context", 1 },
+				{ "wa batchbuffer", "WA batch", 1 },
+				{ "user", "user", 0 },
+				{ "semaphores", "semaphores", 0 },
+				{ "guc log buffer", "GuC log", 0 },
+				{ },
+			}, *b;
+			char *new_ring_name;
+
+			new_ring_name = malloc(dashes - line);
+			strncpy(new_ring_name, line, dashes - line);
+			new_ring_name[dashes - line - 1] = '\0';
+
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
+			gtt_offset = 0;
+			head_offset = -1;
+
+			free(ring_name);
+			ring_name = new_ring_name;
+
+			dashes += 4;
+			for (b = buffers; b->match; b++) {
+				uint32_t lo, hi;
+
+				if (strncasecmp(dashes, b->match,
+						strlen(b->match)))
+					continue;
+
+				dashes = strchr(dashes, '=');
+				if (!dashes)
+					break;
+
+				matched = sscanf(dashes, "= 0x%08x %08x\n",
+						 &hi, &lo);
+				if (matched > 0) {
+					gtt_offset = hi;
+					if (matched == 2) {
+						gtt_offset <<= 32;
+						gtt_offset |= lo;
+					}
+				}
+
+				do_decode = b->do_decode;
+				buffer_name = b->name;
+				if (b == buffers)
+					head_offset = head[head_idx++];
+				break;
+			}
+
 			continue;
 		}
 
@@ -662,7 +659,7 @@ read_data_file(FILE *file)
 			decode(decode_ctx,
 			       buffer_name, ring_name,
 			       gtt_offset, head_offset,
-			       data, &count);
+			       data, &count, do_decode);
 
 			printf("%s", line);
 
@@ -745,13 +742,37 @@ read_data_file(FILE *file)
 	decode(decode_ctx,
 	       buffer_name, ring_name,
 	       gtt_offset, head_offset,
-	       data, &count);
+	       data, &count, do_decode);
 
 	free(data);
 	free(line);
 	free(ring_name);
 }
 
+static void setup_pager(void)
+{
+	int fds[2];
+
+	if (pipe(fds) == -1)
+		return;
+
+	switch (fork()) {
+	case -1:
+		break;
+	case 0:
+		close(fds[1]);
+		dup2(fds[0], 0);
+		execlp("less", "less", "-FRSi", NULL);
+		break;
+
+	default:
+		close(fds[0]);
+		dup2(fds[1], 1);
+		close(fds[1]);
+		break;
+	}
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -777,6 +798,9 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (isatty(1))
+		setup_pager();
+
 	if (argc == 1) {
 		if (isatty(0)) {
 			path = "/sys/class/drm/card0/error";
